@@ -19,6 +19,7 @@
  *   Christopher Blizzard <blizzard@mozilla.org>
  */
 
+#include <mozilla-config.h>
 #include <nsServiceManagerUtils.h>
 #include <nsIComponentManager.h>
 
@@ -208,7 +209,7 @@ EmbedPrivate::Realize(PRBool *aAlreadyRealized)
   mNavigation->SetSessionHistory(mSessionHistory);
 
   // create the window
-  mWindow->CreateWindow();
+  mWindow->CreateEmbedWindow();
 
   // bind the progress listener to the browser object
   nsCOMPtr<nsISupportsWeakReference> supportsWeak;
