@@ -22,10 +22,8 @@
 #define __KZ_MOZ_UTILS_H__
 
 #include <gtk/gtk.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIComponentManager.h>
 #include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
 #include <nsIDOMWindow.h>
 #include <nsIURI.h>
 #include <nsIIOService.h>
@@ -42,10 +40,10 @@
 #include <nsEmbedString.h>
 #undef MOZILLA_STRICT_API
 
-	GtkWidget *GetGtkWindowForDOMWindow (nsIDOMWindow* aDOMWindow);
+GtkWidget *GetGtkWindowForDOMWindow (nsIDOMWindow* aDOMWindow);
 
-	gboolean  NewURI		    (nsIURI **result,
-    					     const char *spec);
+gboolean  NewURI		    (nsIURI **result,
+				     const char *spec);
 
 /* 
  * The following functions is taken from include/necko/nsNetUtils.h in mozilla
@@ -67,20 +65,6 @@ GetIOService (nsIIOService **ioService)
 }
 
 
-inline nsresult
-NS_NewURI(nsIURI **result, 
-          const nsACString &spec, 
-          const char *charset = nsnull,
-          nsIURI *baseURI = nsnull)
-{
-    nsresult rv;
-    nsCOMPtr<nsIIOService> is;
-    rv = GetIOService(getter_AddRefs(is));
-    if (is)
-        rv = is->NewURI(spec, charset, baseURI, result);
-    return rv; 
-}
-
 inline nsresult
 NS_NewFileURI(nsIURI* *result, 
               nsIFile* spec)
@@ -190,22 +174,6 @@ NS_NewLocalFileOutputStream(nsIOutputStream **aResult,
     return rv;
 }
 
-inline nsresult
-NS_NewLoadGroup(nsILoadGroup      **result,
-                nsIRequestObserver *obs)
-{
-    nsresult rv;
-    static NS_DEFINE_CID(kLoadGroupCID, NS_LOADGROUP_CID);
-    nsCOMPtr<nsILoadGroup> group =
-        do_CreateInstance(kLoadGroupCID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-        rv = group->SetGroupObserver(obs);
-        if (NS_SUCCEEDED(rv))
-            NS_ADDREF(*result = group);
-    }
-    return rv;
-}
-
 inline void
 ToLowerCase (nsAString &aString)
 {
@@ -226,6 +194,7 @@ nsEscapeHTML2(const PRUnichar *aSourceBuffer, PRInt32 aSourceBufferLen)
 {
   // if the caller didn't calculate the length
   if (aSourceBufferLen == -1) {
+    //aSourceBufferLen = NS_strlen(aSourceBuffer); // ...then I will
     aSourceBufferLen = nsCRT::strlen(aSourceBuffer); // ...then I will
   }
 
