@@ -31,6 +31,7 @@
 #include "kz-proxy-item.h"
 #include "gtk-utils.h"
 #include "kz-missing.h"
+#include "kz-bookmark.h"
 
 typedef struct _KzIEPrivate	KzIEPrivate;
 struct _KzIEPrivate
@@ -73,13 +74,11 @@ static gboolean	    kz_ie_embed_dom_mouse_click  (GtkIEEmbed   *embed,
 static gboolean	    kz_ie_embed_new_window	 (GtkIEEmbed   *embed, GtkIEEmbed **newembed);
 static gboolean	    kz_ie_embed_close_window	 (GtkIEEmbed   *embed);
 static void	    kz_ie_embed_selection_changed(GtkIEEmbed   *embed);
-
 /* embed iface */
 static void         kz_ie_load_url         (KzEmbed      *kzembed,
 					    const gchar  *url);
 static const gchar *kz_ie_get_title        (KzEmbed      *kzembed);
 static const gchar *kz_ie_get_location     (KzEmbed      *kzembed);
-static gchar	   *kz_ie_ensure_title 	   (KzEmbed	 *kzembed);
 static gdouble	    kz_ie_get_progress	   (KzEmbed	 *kzembed);
 static void         kz_ie_reload           (KzEmbed      *kzembed,
 					    KzEmbedReloadFlag flags);
@@ -101,7 +100,7 @@ static void         kz_ie_select_all       (KzEmbed      *kzembed);
 static void         kz_ie_print            (KzEmbed      *kzembed);
 static void         kz_ie_print_preview    (KzEmbed      *kzembed);
 static void	    kz_ie_set_text_size	   (KzEmbed      *kzembed, 
-					    int           zoom, 
+					    gint          zoom, 
 					    gboolean      reflow);
 static gint	    kz_ie_get_text_size	   (KzEmbed      *kzembed);
 static void	    kz_ie_set_encoding	   (KzEmbed      *kzembed,
@@ -114,11 +113,27 @@ static guint	    kz_ie_get_last_modified
 static gchar       *kz_ie_get_selection_string 
 					   (KzEmbed      *kzembed);
 static gboolean	    kz_ie_find		   (KzEmbed	 *kzembed,
-					    const char   *keyword,
+					    const gchar  *keyword,
 					    gboolean      backward);
 static gboolean     kz_ie_get_lock         (KzEmbed      *kzembed);
 static void         kz_ie_set_lock         (KzEmbed      *kzembed,
 					    gboolean      lock);
+static void	    get_history            (KzEmbed      *kzembed,
+					    KzBookmarkFolder   *history);
+static void	    set_history            (KzEmbed      *kzembed,
+					    KzBookmarkFolder   *history);
+static void         copy_page              (KzEmbed      *src_embed,
+					    KzEmbed      *dest_embed);
+static gboolean     shistory_get_pos       (KzEmbed      *kzembed,
+					    gint         *pos,
+					    gint         *count);
+static void         shistory_get_nth       (KzEmbed      *kzembed, 
+					    gint          nth,
+					    gboolean      is_relative,
+					    gchar       **uri,
+					    gchar       **title);
+static void         go_history_index       (KzEmbed      *kzembed,
+					    gint          index);
 
 /* KzEmbedPrefs interfaces */
 /* no functions is implemented yet */
@@ -213,10 +228,11 @@ kz_ie_iface_init (KzEmbedIFace *iface)
 {
 	iface->load_url               = kz_ie_load_url;
 	iface->view_source            = NULL;
+	iface->view_current_page_source_in_new
+				      = NULL;
 	iface->is_loading             = kz_ie_is_loading;
 	iface->get_title              = kz_ie_get_title;
 	iface->get_location           = kz_ie_get_location;
-	iface->ensure_title           = kz_ie_ensure_title;
 	iface->get_link_message       = NULL;
 	iface->get_progress           = kz_ie_get_progress;
 	iface->can_cut_selection      = kz_ie_can_cut_selection;
@@ -231,10 +247,9 @@ kz_ie_iface_init (KzEmbedIFace *iface)
 	iface->incremental_search     = NULL;
 	iface->selection_is_collapsed = NULL;
 	iface->get_links              = NULL;
-	iface->copy_page              = NULL;
-	iface->shistory_copy          = NULL;
-	iface->shistory_get_pos       = NULL;
-	iface->shistory_get_nth       = NULL;
+	iface->copy_page              = copy_page;
+	iface->shistory_get_pos       = shistory_get_pos;
+	iface->shistory_get_nth       = shistory_get_nth;
 	iface->reload                 = kz_ie_reload;
 	iface->stop_load              = kz_ie_stop_load;
 	iface->go_back                = kz_ie_go_back;
@@ -249,7 +264,7 @@ kz_ie_iface_init (KzEmbedIFace *iface)
 	iface->get_nav_link           = NULL;
 	iface->get_nth_nav_link       = NULL;
 	iface->get_nav_links          = NULL;
-	iface->go_history_index       = NULL;
+	iface->go_history_index       = go_history_index;
 	iface->do_command             = NULL;
 	iface->can_do_command         = NULL;
 	iface->get_lock               = kz_ie_get_lock;
@@ -272,8 +287,8 @@ kz_ie_iface_init (KzEmbedIFace *iface)
 	iface->set_text_size          = kz_ie_set_text_size;
 	iface->get_text_size          = kz_ie_get_text_size;
 	iface->get_html_with_contents = NULL;
-	iface->set_history            = NULL;
-	iface->get_history            = NULL;
+	iface->set_history            = set_history;
+	iface->get_history            = get_history;
 	iface->get_last_modified      = kz_ie_get_last_modified;
 	iface->fine_scroll            = NULL;
 	iface->page_up                = NULL;
@@ -368,7 +383,7 @@ kz_ie_embed_location (GtkIEEmbed *ieembed, const gchar  *location)
 	if (priv->location)
 		g_free(priv->location);
 	priv->location = g_strdup(location);
-	g_signal_emit_by_name(ieembed, "kz-location");
+	g_signal_emit_by_name(ieembed, "kz-location", location);
 }
 
 static void
@@ -379,7 +394,7 @@ kz_ie_embed_title (GtkIEEmbed *ieembed, const gchar *title)
 	if (priv->title)
 		g_free(priv->title);
 	priv->title = g_strdup(title);
-	g_signal_emit_by_name(ieembed, "kz-title");
+	g_signal_emit_by_name(ieembed, "kz-title", title);
 }
 
 static void
@@ -388,10 +403,20 @@ kz_ie_embed_progress (GtkIEEmbed *ieembed,
 		      glong max_progress)
 {
 	KzIEPrivate *priv = KZ_IE_GET_PRIVATE(ieembed);
+	gdouble progress = 0.0;
+
 	priv->current_progress = current_progress;
 	priv->max_progress = max_progress;
 
-	g_signal_emit_by_name(ieembed, "kz-progress");
+	if (priv->max_progress > 0 ||
+	    priv->current_progress >= 0)
+	{
+		progress = (gdouble) priv->current_progress
+			/ (gdouble) priv->max_progress;
+	}
+
+
+	g_signal_emit_by_name(ieembed, "kz-progress", progress);
 }
 
 static void
@@ -428,10 +453,12 @@ create_kz_embed_event_mouse (GtkIEEmbedDOMMouseEvent *event)
 	kzevent->button = event->button;
 	kzevent->cinfo.context = KZ_CONTEXT_NONE;
 
-	if (event->target) {
+	if (event->target)
+	{
 		const gchar *name;
 		name = gtk_ie_embed_dom_event_target_get_name(event->target);
-		if (!strcmp("a", name)) {
+		if (!g_ascii_strcasecmp("a", name))
+		{
 			const gchar *value;
 			kzevent->cinfo.context = KZ_CONTEXT_LINK;
 			value = gtk_ie_embed_dom_event_target_get_attribute_value(event->target,
@@ -525,22 +552,6 @@ kz_ie_get_location (KzEmbed *kzembed)
 	return KZ_IE_GET_PRIVATE(kzembed)->location;
 }
 
-static gchar *
-kz_ie_ensure_title (KzEmbed *kzembed)
-{
-	const gchar *title;
-
-	title = kz_ie_get_title(kzembed);
-	if (title)
-		return g_strdup(title);
-
-	title = kz_ie_get_location(kzembed);
-	if (title)
-		return g_strdup(title);
-
-	return g_strdup(_("No title"));
-}
-
 static gdouble
 kz_ie_get_progress (KzEmbed *kzembed)
 {
@@ -817,3 +828,189 @@ kz_ie_set_lock (KzEmbed *kzembed, gboolean lock)
 	priv->lock = lock;
 }
 
+static void
+merge_to_bookmark_list (const gchar *uri, const gchar *title,
+			GList **bookmark_list_node, KzBookmarkFolder *folder)
+{
+	if (*bookmark_list_node)
+	{
+		kz_bookmark_set_title(KZ_BOOKMARK((*bookmark_list_node)->data), title);
+		kz_bookmark_set_link(KZ_BOOKMARK((*bookmark_list_node)->data), uri);
+		*bookmark_list_node = g_list_next(*bookmark_list_node);
+	}
+	else
+	{
+		KzBookmark *bookmark;
+		bookmark = kz_bookmark_new_with_attrs(title, uri, NULL);
+		kz_bookmark_folder_append(folder, bookmark);
+		g_object_unref(bookmark);
+	}
+}
+
+static guint
+merge_ie_history_list_to_bookmark_list (const GList *history_list, GList **bookmark_list_node,
+					KzBookmarkFolder *folder)
+{
+	const GList *node;
+	guint count = 0;
+
+	for (node = history_list; node; node = g_list_next(node))
+	{
+		const gchar *uri, *title;
+		GtkIEEmbedHistoryItem *item = GTK_IE_EMBED_HISTORY_ITEM(node->data);
+
+		uri = gtk_ie_embed_history_item_get_uri(item);
+		title = gtk_ie_embed_history_item_get_title(item);
+		merge_to_bookmark_list(uri, title, bookmark_list_node, folder);
+		count++;
+	}
+	return count;
+}
+
+static void
+history_list_free (GList *list)
+{
+	g_list_foreach(list, (GFunc)g_object_unref, NULL);
+	g_list_free(list);
+}
+
+static void
+get_history (KzEmbed *kzembed, KzBookmarkFolder *history)
+{
+	GList *children, *bookmark_node;
+	GList *backward_history, *forward_history;
+	gint current_position = 0;
+	const gchar *uri, *title;
+
+	backward_history = gtk_ie_embed_get_backward_history(GTK_IE_EMBED(kzembed));
+	forward_history = gtk_ie_embed_get_forward_history(GTK_IE_EMBED(kzembed));
+
+	children = kz_bookmark_folder_get_children(history);
+	bookmark_node = children;
+	current_position = merge_ie_history_list_to_bookmark_list(backward_history,
+							          &bookmark_node, 
+								  history);
+
+	uri = KZ_IE_GET_PRIVATE(kzembed)->location;
+	title = KZ_IE_GET_PRIVATE(kzembed)->title;
+	merge_to_bookmark_list(uri, title, &bookmark_node, history);
+
+	merge_ie_history_list_to_bookmark_list(forward_history,
+					       &bookmark_node,
+					       history);
+
+	if (children)
+		g_list_free(children);
+	if (backward_history)
+		history_list_free(backward_history);
+	if (forward_history)
+		history_list_free(forward_history);
+
+	kz_bookmark_folder_set_current_position(history, current_position);
+}
+
+static void
+append_history_item (KzBookmark *bookmark, GList **history_item_list)
+{
+        GtkIEEmbedHistoryItem *item;
+
+        item = gtk_ie_embed_history_item_new(kz_bookmark_get_link(bookmark),
+					     kz_bookmark_get_title(bookmark));
+        *history_item_list =
+                g_list_append(*history_item_list, item);
+}
+
+static GList *
+bookmark_to_history_item_list (KzBookmarkFolder *history)
+{
+	GList *history_item_list = NULL;
+
+	kz_bookmark_folder_foreach_child(history, 
+					 (GFunc)append_history_item, &history_item_list);
+
+	return history_item_list;
+}
+
+static void
+set_history (KzEmbed *kzembed, KzBookmarkFolder *history)
+{
+	GList *history_item_list;
+	gint current_position;
+
+	current_position = kz_bookmark_folder_get_current_position(history);
+
+	history_item_list = bookmark_to_history_item_list(history);
+	if (!history_item_list)
+		return;
+
+	gtk_ie_embed_set_whole_history(GTK_IE_EMBED(kzembed),
+				       history_item_list, current_position);
+	history_list_free(history_item_list);
+}
+
+static void
+copy_page (KzEmbed *src_embed, KzEmbed *dest_embed)
+{
+        GList *backward_history_list, *forward_history_list;
+        GList *history_list = NULL;
+        GtkIEEmbedHistoryItem *current_item;
+        gint current_position;
+        GtkIEEmbed *src_ie = GTK_IE_EMBED(src_embed);
+
+        backward_history_list = gtk_ie_embed_get_backward_history(src_ie);
+        forward_history_list = gtk_ie_embed_get_forward_history(src_ie);
+        current_item = gtk_ie_embed_get_history_item_at_relative_position(src_ie, 0);
+        current_position = backward_history_list ? g_list_length(backward_history_list) : 0;
+
+        if (current_item)
+                history_list = g_list_append(backward_history_list, current_item);
+        history_list = g_list_concat(history_list, forward_history_list);
+
+	gtk_ie_embed_set_whole_history(GTK_IE_EMBED(dest_embed),
+				       history_list, current_position + 1);
+        history_list_free(history_list);
+        if (current_item)
+                g_object_unref(current_item);
+}
+
+static gboolean 
+shistory_get_pos (KzEmbed *kzembed, gint *pos, gint *count)
+{
+	GtkIEEmbed *ie = GTK_IE_EMBED(kzembed);
+
+	*pos = (gint)gtk_ie_embed_get_current_position_in_history(ie);
+	*count = (gint)gtk_ie_embed_get_history_count(ie);
+
+	return TRUE;
+}
+
+static void
+shistory_get_nth (KzEmbed *kzembed,
+		  gint nth, gboolean is_relative,
+		  gchar **uri, gchar **title)
+{
+        GtkIEEmbedHistoryItem *item;
+	guint current_position;
+
+	current_position = gtk_ie_embed_get_current_position_in_history(GTK_IE_EMBED(kzembed));
+	if (!is_relative)
+		nth -= current_position;
+	item = gtk_ie_embed_get_history_item_at_relative_position(GTK_IE_EMBED(kzembed), nth);
+	if (!item)
+		return;
+
+	*uri = g_strdup(gtk_ie_embed_history_item_get_uri(item));
+	*title = g_strdup(gtk_ie_embed_history_item_get_title(item));
+	g_object_unref(item);
+}
+
+static void
+go_history_index (KzEmbed *kzembed, gint index)
+{
+	guint current_position;
+
+	current_position = gtk_ie_embed_get_current_position_in_history(GTK_IE_EMBED(kzembed));
+	gtk_ie_embed_go_relative_position(GTK_IE_EMBED(kzembed), index - current_position);
+}
+
+
