@@ -104,7 +104,7 @@ open_location (GtkAction *action, KzWindow *kz, LocationType location, WindowTyp
 		break;
 	case NEW_TAB:
 		kz_window_open_new_tab_with_parent(kz, uri,
-						   KZ_WINDOW_CURRENT_PAGE(kz));
+						   GTK_WIDGET(KZ_WINDOW_CURRENT_EMBED(kz)));
 		break;
 	case NEW_WINDOW:
 	{
@@ -293,8 +293,7 @@ static void
 act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 {
 	const KzEmbedEventMouse *event;
-	GtkWidget *widget;
-
+	KzEmbed *embed;
 	GPid pid;
 	GSpawnFlags flags;
 	gint argc;
@@ -303,8 +302,9 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	EditorInfo *einfo = NULL;
 	
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	widget = KZ_WINDOW_CURRENT_PAGE(kz);
-	g_return_if_fail(KZ_EMBED(widget));
+	embed = KZ_WINDOW_CURRENT_EMBED(kz);
+	if (!embed)
+		return;
 
 	event = kz_window_get_mouse_event_info(kz);
 	g_return_if_fail(event);
@@ -316,17 +316,18 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	if (event->cinfo.context & KZ_CONTEXT_TEXTAREA)
 	{
 		einfo = g_new0(EditorInfo, 1);
-		einfo->kzembed  = g_object_ref(KZ_EMBED(widget));
+		einfo->kzembed  = g_object_ref(embed);
 		einfo->element  = event->cinfo.element;
-		text = kz_embed_get_text_from_textarea(KZ_EMBED(widget),
+		text = kz_embed_get_text_from_textarea(embed,
 						       einfo->element);
 		/* text in textare store in temporary file */
 		if (text)
 		{
+			ssize_t size;
 			gint fd = g_file_open_tmp("kzXXXXXX",
 						  &einfo->filename,
 						  NULL);
-			write(fd, text, strlen(text));
+			size = write(fd, text, strlen(text));
 			close(fd);
 			g_free(text);
 		}
@@ -339,9 +340,9 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	}
 	
 	/* */
-	g_signal_connect(widget, "kz-net-start",
+	g_signal_connect(embed, "kz-net-start",
 			 G_CALLBACK(cb_embed_changed), einfo);
-	g_signal_connect(widget, "destroy",
+	g_signal_connect(embed, "destroy",
 			 G_CALLBACK(cb_embed_changed), einfo);
 	/* launch editor */
 	g_shell_parse_argv(command,
@@ -510,22 +511,21 @@ cb_encoding_menuitem_destroy (GtkWidget *menuitem, gchar *encoding)
 static void
 cb_encoding_menuitem_activate (GtkWidget *menuitem, KzWindow *kz)
 {
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+	KzEmbed *embed = KZ_WINDOW_CURRENT_EMBED(kz);
 
-	if (GTK_CHECK_MENU_ITEM(menuitem)->active && KZ_EMBED(widget))
+	if (GTK_CHECK_MENU_ITEM(menuitem)->active && embed)
 	{
 		const gchar* code
 			= g_object_get_data(G_OBJECT(menuitem),
 					    KZ_ACTIONS_POPUP_LANGUAGE_KEY);
 		GtkAction *action;
 
-		kz_embed_set_encoding(KZ_EMBED(widget), code);
+		kz_embed_set_encoding(embed, code);
 
 		action = gtk_action_group_get_action(kz->actions, "Reload");
 
 		gtk_action_activate(action);
 	}
-	return;
 }
 
 static GtkWidget *
@@ -608,7 +608,7 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	GtkWidget *encoding_submenu;
 	GtkWidget *menuitem;
 	GSList *group = NULL;
-	GtkWidget *page = KZ_WINDOW_CURRENT_PAGE(kz);
+	KzEmbed *embed = KZ_WINDOW_CURRENT_EMBED(kz);
 	gchar *encodings_xml;
 	gchar *current_encoding = NULL;
 	gboolean forced = FALSE;
@@ -628,9 +628,9 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	encoding_submenu = gtk_menu_new();
 
 	/* get current charset */
-	if(KZ_EMBED(page))
+	if(embed)
 	{
-		kz_embed_get_encoding(KZ_EMBED(page),
+		kz_embed_get_encoding(embed,
 				      &current_encoding, &forced);
 	}
 
@@ -678,7 +678,7 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 			gtk_widget_show(menuitem);
 		}
 	}
-	g_object_unref(G_OBJECT(xml));
+	g_object_unref(xml);
 
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(encoding_menu),
 				  encoding_submenu);
@@ -697,7 +697,6 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 {
 	gint i;
 	GtkWidget *tablist_submenu;
-	GtkWidget *page;
 	gint current_page_num;
 	KzFavicon *kzfav = KZ_GET_FAVICON;
 	gchar *color = NULL;
@@ -752,65 +751,62 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 		tablist_submenu = tablist_menu;
 	}
 
-	page = KZ_WINDOW_CURRENT_PAGE(kz);
-	current_page_num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
-						GTK_WIDGET(page));
+	current_page_num = kz_notebook_get_current_page(KZ_NOTEBOOK(kz->notebook));
 
 	for (i = 0; i < num; i++)
 	{
-		KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_NTH_PAGE(kz, i));
+		KzEmbed *kzembed = KZ_WINDOW_NTH_EMBED(kz, i);
 		GtkWidget *favicon;
+		gchar *title, *escaped, *markup_title;
+		GtkWidget *menuitem;
+		KzTabLabel *tab;
+		KzTabLabelState state;
 
-		if (KZ_EMBED(kzembed))
+		if (!kzembed)
+                        continue;
+			
+		tab = kz_window_get_tab_label(kz, kzembed);
+		state = kz_tab_label_get_state(tab);
+		title = kz_embed_ensure_title(kzembed);
+		escaped = g_markup_escape_text(title,
+					       strlen(title));
+		menuitem = gtk_image_menu_item_new_with_label(title);
+
+		if (i == current_page_num)
 		{
-			gchar *title, *escaped, *markup_title;
-			GtkWidget *menuitem;
-			GtkWidget *tab;
-			KzTabLabelState state;
-			
-			tab = kz_window_get_tab_label(kz, GTK_WIDGET(kzembed));
-			state = kz_tab_label_get_state(KZ_TAB_LABEL(tab));
-			title = kz_embed_ensure_title(kzembed);
-			escaped = g_markup_escape_text(title,
-						       strlen(title));
-			menuitem = gtk_image_menu_item_new_with_label(title);
-
-			if (i == current_page_num)
-			{
-				markup_title = g_strdup_printf("<b>%s</b>",
-							       escaped);
-			}
-			else
-			{
-				markup_title = g_strdup_printf("<span foreground=\"%s\">%s</span>",
-							       label_color[state],
-							       escaped);
-			}
-
-			gtk_label_set_markup(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menuitem))),
-					     markup_title);
-			g_free(markup_title);
-			g_free(escaped);
-			
-			/* favicon */
-			favicon = kz_favicon_get_widget(kzfav,
-							kz_embed_get_location(kzembed),
-							KZ_ICON_SIZE_BOOKMARK_MENU);
-			if (favicon)
-			{
-				gtk_widget_show(favicon);
-				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),
-							      favicon);
-			}
-
-			g_object_set_data (G_OBJECT(menuitem), KZ_ACTIONS_POPUP_TAB_KEY, kzembed);
-
-			g_signal_connect(menuitem, "activate",
-					 G_CALLBACK(cb_tablist_menuitem_activate), kz);
-			gtk_menu_shell_append(GTK_MENU_SHELL(tablist_submenu), menuitem);
-			gtk_widget_show(menuitem);
-			g_free(title);
+			markup_title = g_strdup_printf("<b>%s</b>",
+						       escaped);
 		}
+		else
+		{
+			markup_title = g_strdup_printf("<span foreground=\"%s\">%s</span>",
+						       label_color[state],
+						       escaped);
+		}
+
+		gtk_label_set_markup(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menuitem))),
+				     markup_title);
+		g_free(markup_title);
+		g_free(escaped);
+		
+		/* favicon */
+		favicon = kz_favicon_get_widget(kzfav,
+						kz_embed_get_location(kzembed),
+						KZ_ICON_SIZE_BOOKMARK_MENU);
+		if (favicon)
+		{
+			gtk_widget_show(favicon);
+			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),
+						      favicon);
+		}
+
+		g_object_set_data (G_OBJECT(menuitem), KZ_ACTIONS_POPUP_TAB_KEY, kzembed);
+
+		g_signal_connect(menuitem, "activate",
+				 G_CALLBACK(cb_tablist_menuitem_activate), kz);
+		gtk_menu_shell_append(GTK_MENU_SHELL(tablist_submenu), menuitem);
+		gtk_widget_show(menuitem);
+		g_free(title);
 	}
 	if (GTK_IS_MENU_ITEM(tablist_menu))
 	{
@@ -893,7 +889,10 @@ kz_actions_popup_menu_modal (KzWindow *kz, guint button, guint time)
 	if (extra_menu)
 	{
 		KzTabLabel *kztab;
-		kztab = KZ_TAB_LABEL(kz_window_get_tab_label(kz, KZ_WINDOW_CURRENT_PAGE(kz)));
+		gint current_page;
+
+		current_page = kz_notebook_get_current_page(KZ_NOTEBOOK(kz->notebook));
+		kztab = kz_notebook_get_nth_tab_label(KZ_NOTEBOOK(kz->notebook), current_page);
 		kz_actions_dynamic_append_copy_in_user_format_menuitem
 			(kztab, GTK_MENU_ITEM(extra_menu));
 	}
