@@ -164,18 +164,15 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 static void
 activate (GtkAction *action)
 {
-	GtkWidget *widget;
+	KzEmbed *embed;
 
 	g_return_if_fail(KZ_NAVIGATION_ACTION(action));
 
-	widget = KZ_WINDOW_CURRENT_PAGE(KZ_NAVIGATION_ACTION(action)->kz);
+	embed = KZ_WINDOW_CURRENT_EMBED(KZ_NAVIGATION_ACTION(action)->kz);
 
-	if (!widget) return;
+	if (!embed) return;
 
-	if (KZ_EMBED(widget))
-	{
-		kz_embed_go_back(KZ_EMBED(widget));
-	}
+	kz_embed_go_back(embed);
 }
 
 static void
@@ -183,7 +180,7 @@ cb_menu_item_activate (GtkWidget *menu, KzNavigationAction *action)
 {
 	gint go_nth;
 
-	KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(action->kz));
+	KzEmbed *kzembed = KZ_WINDOW_CURRENT_EMBED(action->kz);
 
 	go_nth = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "go_nth"));
 
@@ -201,7 +198,7 @@ cb_menu_activated (GtkMenuToolButton *b, KzNavigationAction *action)
 {
 	int i, pos, count;
 	GtkWidget *menu;
-	KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(action->kz));
+	KzEmbed *kzembed = KZ_WINDOW_CURRENT_EMBED(action->kz);
 	KzFavicon *kzfav = KZ_GET_FAVICON;
 
 	menu = gtk_menu_tool_button_get_menu(b);
