@@ -29,10 +29,10 @@
 #include "kz-smart-bookmark.h"
 #include "utils/utils.h"
 #include "utils/gtk-utils.h"
-#include "gobject-utils.h"
 #include "kz-entry.h"
 #include "kz-actions.h"
 #include "kz-icons.h"
+#include "kz-bookmark-folder.h"
 
 #define KZ_SMART_BOOKMARK_ACTION_BOOKMARK_KEY "KzSmartBookmarkAction::Bookmark"
 
@@ -42,19 +42,17 @@ enum {
 	PROP_KZ_BOOKMARK
 };
 
-static void kz_smart_bookmark_action_class_init   (KzSmartBookmarkActionClass *class);
-static void kz_smart_bookmark_action_init         (KzSmartBookmarkAction *action);
-static void kz_smart_bookmark_action_dispose      (GObject *obj);
-static void kz_smart_bookmark_action_set_property (GObject      *object,
-						   guint         prop_id,
-						   const GValue *value,
-						   GParamSpec   *pspec);
-static void kz_smart_bookmark_action_get_property (GObject      *object,
-						   guint         prop_id,
-						   GValue       *value,
-						   GParamSpec   *pspec);
+static void dispose      (GObject *obj);
+static void set_property (GObject      *object,
+                          guint         prop_id,
+                          const GValue *value,
+                          GParamSpec   *pspec);
+static void get_property (GObject      *object,
+                          guint         prop_id,
+                          GValue       *value,
+                          GParamSpec   *pspec);
 
-static void kz_smart_bookmark_action_activate     (GtkAction    *action);
+static void activate     (GtkAction    *action);
 
 static GtkWidget *create_tool_item                (GtkAction    *action);
 static void       connect_proxy                   (GtkAction    *action,
@@ -76,12 +74,12 @@ static void       cb_menuitem_activate            (GtkWidget *menuitem,
 static void       cb_bookmark_notify             (GObject     *object,
 						  GParamSpec  *pspec,
 	  			    	 	  KzSmartBookmarkAction *action);
-static void       cb_bookmark_updated            (KzBookmark *folder,
+static void       cb_bookmark_updated            (KzBookmarkFolder *folder,
 						  KzSmartBookmarkAction *action);
-static void       cb_bookmark_remove_child       (KzBookmark *folder,
+static void       cb_bookmark_remove_child       (KzBookmarkFolder *folder,
 						  KzBookmark *child,
 	  			    	 	  KzSmartBookmarkAction *action);
-static void       cb_bookmark_insert_child  	 (KzBookmark *folder,
+static void       cb_bookmark_insert_child  	 (KzBookmarkFolder *folder,
 						  KzBookmark *child,
 						  KzBookmark *sibling,
 	  			    	 	  KzSmartBookmarkAction *action);
@@ -91,15 +89,7 @@ static gchar    **get_smart_uris                 (KzSmartBookmarkAction *action)
 
 static void       update_icon_and_bgtext         (KzSmartBookmarkAction *action);
 
-static KzEntryActionClass *parent_class = NULL;
-
-KZ_OBJECT_GET_TYPE(kz_smart_bookmark_action,
-		   "KzSmartBookmarkAction",
-		   KzSmartBookmarkAction,
-		   kz_smart_bookmark_action_class_init,
-		   kz_smart_bookmark_action_init,
-		   KZ_TYPE_ENTRY_ACTION)
-
+G_DEFINE_TYPE(KzSmartBookmarkAction, kz_smart_bookmark_action, KZ_TYPE_ENTRY_ACTION)
 
 static void
 kz_smart_bookmark_action_class_init (KzSmartBookmarkActionClass *klass)
@@ -108,16 +98,15 @@ kz_smart_bookmark_action_class_init (KzSmartBookmarkActionClass *klass)
 	GtkActionClass *action_class;
 	KzEntryActionClass *entry_class;
 
-	parent_class  = g_type_class_peek_parent(klass);
 	object_class  = G_OBJECT_CLASS(klass);
 	action_class  = GTK_ACTION_CLASS(klass);
 	entry_class   = KZ_ENTRY_ACTION_CLASS(klass);
 
-	object_class->set_property     = kz_smart_bookmark_action_set_property;
-	object_class->get_property     = kz_smart_bookmark_action_get_property;
-	object_class->dispose          = kz_smart_bookmark_action_dispose;
+	object_class->set_property     = set_property;
+	object_class->get_property     = get_property;
+	object_class->dispose          = dispose;
 
-	action_class->activate         = kz_smart_bookmark_action_activate;
+	action_class->activate         = activate;
 	action_class->connect_proxy    = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
 	action_class->create_tool_item = create_tool_item;
@@ -154,7 +143,7 @@ kz_smart_bookmark_action_init (KzSmartBookmarkAction *action)
 
 
 static void
-kz_smart_bookmark_action_dispose (GObject *obj)
+dispose (GObject *obj)
 {
 	KzSmartBookmarkAction *action = KZ_SMART_BOOKMARK_ACTION(obj);
 
@@ -188,19 +177,19 @@ kz_smart_bookmark_action_dispose (GObject *obj)
 	if (action->menulist)
 	{
 		g_list_free(action->menulist);
-		action->menu = NULL;
+		action->menulist = NULL;
 	}
 
-	if (G_OBJECT_CLASS(parent_class)->dispose)
-		G_OBJECT_CLASS(parent_class)->dispose(obj);
+	if (G_OBJECT_CLASS(kz_smart_bookmark_action_parent_class)->dispose)
+		G_OBJECT_CLASS(kz_smart_bookmark_action_parent_class)->dispose(obj);
 }
 
 
 static void
-kz_smart_bookmark_action_set_property (GObject         *object,
-				       guint            prop_id,
-				       const GValue    *value,
-				       GParamSpec      *pspec)
+set_property (GObject         *object,
+              guint            prop_id,
+              const GValue    *value,
+              GParamSpec      *pspec)
 {
 	KzSmartBookmarkAction *action = KZ_SMART_BOOKMARK_ACTION(object);
   
@@ -214,7 +203,7 @@ kz_smart_bookmark_action_set_property (GObject         *object,
 		/* if bookmark is a folder, connect signals */
 		if (kz_bookmark_is_folder(action->bookmark))
 		{
-			cb_bookmark_updated(action->bookmark, action);
+			cb_bookmark_updated(KZ_BOOKMARK_FOLDER(action->bookmark), action);
 	/*		g_signal_connect(bookmark, "children-reordered",
 					 G_CALLBACK(cb_bookmark_updated),
 					 action);*/
@@ -234,10 +223,10 @@ kz_smart_bookmark_action_set_property (GObject         *object,
 
 
 static void
-kz_smart_bookmark_action_get_property (GObject         *object,
-				       guint            prop_id,
-				       GValue          *value,
-				       GParamSpec      *pspec)
+get_property (GObject         *object,
+              guint            prop_id,
+              GValue          *value,
+              GParamSpec      *pspec)
 {
 	KzSmartBookmarkAction *action = KZ_SMART_BOOKMARK_ACTION(object);
 
@@ -257,7 +246,7 @@ kz_smart_bookmark_action_get_property (GObject         *object,
 
 
 static void
-kz_smart_bookmark_action_activate (GtkAction *action)
+activate (GtkAction *action)
 {
 	KzSmartBookmarkAction *smart_action;
 	gchar *smart_uri;
@@ -265,8 +254,8 @@ kz_smart_bookmark_action_activate (GtkAction *action)
 
 	g_return_if_fail(KZ_IS_SMART_BOOKMARK_ACTION(action));
 
-	if (GTK_ACTION_CLASS(parent_class)->activate)
-		GTK_ACTION_CLASS(parent_class)->activate(action);
+	if (GTK_ACTION_CLASS(kz_smart_bookmark_action_parent_class)->activate)
+		GTK_ACTION_CLASS(kz_smart_bookmark_action_parent_class)->activate(action);
 
 	smart_action = KZ_SMART_BOOKMARK_ACTION(action);
 
@@ -294,7 +283,7 @@ create_tool_item (GtkAction *action)
 {
 	GtkWidget *widget;
 
-	widget = (*GTK_ACTION_CLASS(parent_class)->create_tool_item) (action);
+	widget = (*GTK_ACTION_CLASS(kz_smart_bookmark_action_parent_class)->create_tool_item) (action);
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM(widget), FALSE);
 
 	return widget;
@@ -306,7 +295,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
 	GtkEntry *entry;
 
-	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
+	GTK_ACTION_CLASS (kz_smart_bookmark_action_parent_class)->connect_proxy (action, proxy);
 
 	entry = kz_entry_action_get_entry_widget(KZ_ENTRY_ACTION(action), proxy);
 	if (entry)
@@ -352,7 +341,7 @@ disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 			 action);
 	}
 
-	GTK_ACTION_CLASS (parent_class)->disconnect_proxy (action, proxy);
+	GTK_ACTION_CLASS (kz_smart_bookmark_action_parent_class)->disconnect_proxy (action, proxy);
 }
 
 
@@ -366,11 +355,11 @@ kz_smart_bookmark_action_new (KzWindow *kz, KzBookmark *bookmark)
 	title = kz_bookmark_get_title(bookmark);
 	link = kz_bookmark_get_link(bookmark);
 
-	if (title && !strncmp(title, "LocationEntry", 13))
+	if (title && g_str_has_prefix(title, "LocationEntry"))
 		name = g_strdup("LocationEntry");
 	else if (link &&
-		!strncmp(link, "history-search:", 15) &&
-		!exists_estindex)
+		g_str_has_prefix(link, "history-search:") &&
+		!KZ_GET_SEARCH)
 	{
 		return NULL;
 	}
@@ -519,7 +508,7 @@ create_menu_item (KzSmartBookmarkAction *action, KzBookmark *bookmark)
 	GtkWidget *menuitem, *image;
 	const gchar *title, *stock_id, *link;
 	GdkPixbuf *favicon;
-	KzFavicon *kzfav;
+	KzFavicon *kzfav = KZ_GET_FAVICON;
 	gboolean is_folder;
 
 	is_folder = kz_bookmark_is_folder(bookmark);
@@ -533,15 +522,13 @@ create_menu_item (KzSmartBookmarkAction *action, KzBookmark *bookmark)
 	menuitem = gtk_image_menu_item_new_with_label(title);
 
 	/* attempt to get favicon */
-	kzfav = kz_favicon_get_instance();
 	favicon = kz_favicon_get_pixbuf(kzfav, link, GTK_ICON_SIZE_MENU);
-	g_object_unref(kzfav);
 
 	if (!favicon)
 	{
-		if (link && !strncmp(link, "history-search:", 15))
+		if (link && g_str_has_prefix(link, "history-search:"))
 			stock_id = KZ_STOCK_HISTORY;
-		else if (link && !strncmp(link, "http://www.amazon.com/", 22))
+		else if (link && g_str_has_prefix(link, "http://www.amazon.com/"))
 			stock_id = KZ_STOCK_BOOK_SEARCH;
 		else
 			stock_id = KZ_STOCK_SEARCH;
@@ -569,49 +556,62 @@ create_menu_item (KzSmartBookmarkAction *action, KzBookmark *bookmark)
 	return menuitem;
 }
 
+static void
+insert_child (KzSmartBookmarkAction *action, KzBookmark *child,
+	      GtkWidget *menuitem, guint pos)
+{
+	gtk_menu_shell_insert(GTK_MENU_SHELL(action->menu),
+			      menuitem, pos);
+	action->menulist = g_list_insert(action->menulist, child, pos);
+	if (pos == 0)
+		update_icon_and_bgtext(action);
+}
 
 static void
-cb_bookmark_updated (KzBookmark *folder, KzSmartBookmarkAction *action)
+cb_bookmark_updated (KzBookmarkFolder *folder, KzSmartBookmarkAction *action)
 {
 	GList *children, *node;
 	GtkWidget *menuitem;
+	int i;
+	guint folder_pos;
 
 	if (action->menu)
 		gtk_widget_destroy(action->menu);
 	action->menu = gtk_menu_new();
 	
 	/* folder menu */
-	menuitem = create_menu_item(action, folder);
+	menuitem = create_menu_item(action, KZ_BOOKMARK(folder));
 	gtk_menu_shell_append(GTK_MENU_SHELL(action->menu),
 			      menuitem);
 	if (action->menulist)
 		g_list_free(action->menulist);
-	action->menulist = g_list_append(action->menulist, folder);
+	action->menulist = g_list_append(NULL, folder);
 
 	update_icon_and_bgtext(action);
-	
+
 	/* children */
-	children = kz_bookmark_get_children(folder);
-	for (node = children; node; node = g_list_next(node))
+	children = kz_bookmark_folder_get_children(folder);
+	folder_pos = kz_bookmark_folder_get_current_position(folder);
+	for (node = children, i = 0; node; node = g_list_next(node), i++)
 	{
-		KzBookmark *child= node->data;
+		KzBookmark *child = node->data;
 
 		if (!KZ_IS_SMART_BOOKMARK(child)) continue;
 
 		menuitem = create_menu_item(action, child);
-		if (menuitem)
-		{
-			gtk_menu_shell_append(GTK_MENU_SHELL(action->menu),
-					      menuitem);
-			g_list_append(action->menulist, child);
-		}
+		if (!menuitem) continue;
+
+		if (i < folder_pos)
+			insert_child(action, child, menuitem, i);
+		else
+			insert_child(action, child, menuitem, i + 1);
 	}
 	g_list_free(children);
 }
 
 
 static void
-cb_bookmark_remove_child (KzBookmark *folder,
+cb_bookmark_remove_child (KzBookmarkFolder *folder,
 			  KzBookmark *child,
 	  		  KzSmartBookmarkAction *action)
 {
@@ -619,7 +619,7 @@ cb_bookmark_remove_child (KzBookmark *folder,
 	GtkWidget *widget;
 	gint pos;
 
-	children = kz_bookmark_get_children(folder);
+	children = kz_bookmark_folder_get_children(folder);
 	if (!children) return;
 
 	menuitems = gtk_container_get_children(GTK_CONTAINER(action->menu));
@@ -636,7 +636,10 @@ cb_bookmark_remove_child (KzBookmark *folder,
 			 G_CALLBACK(cb_menuitem_activate),
 			 child);
 		gtk_widget_destroy(widget);
-	}	
+	}
+
+	if (pos == 0)
+		update_icon_and_bgtext(action);
 
 	g_list_free(menuitems);
 	g_list_free(children);
@@ -644,7 +647,7 @@ cb_bookmark_remove_child (KzBookmark *folder,
 
 
 static void
-cb_bookmark_insert_child (KzBookmark *folder,
+cb_bookmark_insert_child (KzBookmarkFolder *folder,
 			  KzBookmark *child,
 			  KzBookmark *sibling,
 	  		  KzSmartBookmarkAction *action)
@@ -658,26 +661,28 @@ cb_bookmark_insert_child (KzBookmark *folder,
 
 	if (!sibling)
 	{
-		gtk_menu_shell_append(GTK_MENU_SHELL(action->menu),
-				      menuitem);
-		action->menulist = g_list_append(action->menulist, child);
+		guint pos;
+		pos = kz_bookmark_folder_get_child_index(folder, child);
+		if (pos < kz_bookmark_folder_get_current_position(folder))
+			insert_child(action, child, menuitem, pos);
+		else
+			insert_child(action, child, menuitem, pos + 1);
 	}
 	else
 	{
-		GList *children;
 		gint pos;
-		children = kz_bookmark_get_children(folder);
+		pos = kz_bookmark_folder_get_child_index(folder, sibling);
 		
-		pos = g_list_index(children, sibling);
 		gtk_menu_shell_insert(GTK_MENU_SHELL(action->menu),
 				      menuitem, pos - 1);
 		action->menulist = g_list_insert(action->menulist, child, pos - 1);
+		kz_bookmark_folder_set_current_position(folder, pos);
+
 		if (pos == 1)
 		{
 			/* update icon and background text */
 			update_icon_and_bgtext(action);
 		}
-		g_list_free(children);
 	}
 }
 
@@ -685,7 +690,8 @@ cb_bookmark_insert_child (KzBookmark *folder,
 static void
 cb_menuitem_activate (GtkWidget *menuitem, KzSmartBookmarkAction *action)
 {
-	KzBookmark *parent, *bookmark;
+	KzBookmark *bookmark;
+	KzBookmarkFolder *parent;
 
 	bookmark = g_object_get_data(G_OBJECT(menuitem),
 				     KZ_SMART_BOOKMARK_ACTION_BOOKMARK_KEY);
@@ -704,6 +710,7 @@ cb_menuitem_activate (GtkWidget *menuitem, KzSmartBookmarkAction *action)
 						 bookmark);
 		action->menulist = g_list_prepend(action->menulist,
 						  bookmark);
+		kz_bookmark_folder_set_current_position(KZ_BOOKMARK_FOLDER(bookmark), 0);
 		update_icon_and_bgtext(action);
 		return;
 	}
@@ -712,10 +719,14 @@ cb_menuitem_activate (GtkWidget *menuitem, KzSmartBookmarkAction *action)
 
 	/* remove bookmark */
 	g_object_ref(bookmark);
-	kz_bookmark_remove(parent, bookmark);
+	kz_bookmark_folder_remove(parent, bookmark);
 
 	/* insert bookmark */
-	kz_bookmark_prepend(parent, bookmark);
+	kz_bookmark_folder_prepend(parent, bookmark);
+
+	/* now, current bookmark is top of the parent */
+	kz_bookmark_folder_set_current_position(parent,
+				                g_list_index(action->menulist, parent));
 
 	g_object_unref(bookmark);
 }
@@ -779,7 +790,7 @@ get_smart_uri (KzSmartBookmarkAction *action)
 		bookmark = KZ_BOOKMARK(action->menulist->data);
 	else
 		bookmark = action->bookmark;
-	
+
 	uri = kz_entry_action_get_text(KZ_ENTRY_ACTION(action));
 	smart_uri = kz_smart_bookmark_get_smart_uri(KZ_SMART_BOOKMARK(bookmark), uri);
 
@@ -798,7 +809,7 @@ get_smart_uris (KzSmartBookmarkAction *action)
 	
 	g_return_val_if_fail(kz_bookmark_is_folder(bookmark), NULL);
 
-	children = kz_bookmark_get_children(bookmark);
+	children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(bookmark));
 	if (!children) return NULL;
 
 	n_children = g_list_length(children);	
@@ -825,7 +836,7 @@ update_icon_and_bgtext(KzSmartBookmarkAction *action)
 	gboolean is_folder;
 	GSList *node, *proxies;
 	GdkPixbuf *favicon;
-	KzFavicon *kzfav;
+	KzFavicon *kzfav = KZ_GET_FAVICON;
 
 	is_folder = kz_bookmark_is_folder(action->bookmark);
 	if (is_folder)
@@ -837,15 +848,13 @@ update_icon_and_bgtext(KzSmartBookmarkAction *action)
 	link = kz_bookmark_get_link(bookmark);
 
 	/* attempt to get favicon */
-	kzfav = kz_favicon_get_instance();
 	favicon = kz_favicon_get_pixbuf(kzfav, link, GTK_ICON_SIZE_MENU);
-	g_object_unref(kzfav);
 
 	if (!favicon)
 	{
-		if (link && !strncmp(link, "history-search:", 15))
+		if (link && g_str_has_prefix(link, "history-search:"))
 			stock_id = KZ_STOCK_HISTORY;
-		else if (link && !strncmp(link, "http://www.amazon.com/", 22))
+		else if (link && g_str_has_prefix(link, "http://www.amazon.com/"))
 			stock_id = KZ_STOCK_BOOK_SEARCH;
 		else
 			stock_id = KZ_STOCK_SEARCH;
