@@ -18,14 +18,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-file.h 2863 2007-03-14 06:43:16Z ikezoe $
+ *  $Id: kz-bookmark-file.h 3705 2009-02-15 23:23:55Z ikezoe $
  */
 
 #ifndef __KZ_BOOKMARK_FILE_H__
 #define __KZ_BOOKMARK_FILE_H__
 
 #include <glib-object.h>
-#include "kz-bookmark.h"
+#include "kz-bookmark-objects.h"
+#include "kz-bookmark-folder.h"
 
 G_BEGIN_DECLS
 
@@ -38,7 +39,6 @@ G_BEGIN_DECLS
 
 #define KZ_TYPE_BOOKMARK_FILE_TYPE		(kz_bookmark_file_type_get_type())
 
-typedef struct _KzBookmarkFile         KzBookmarkFile;
 typedef struct _KzBookmarkFileClass    KzBookmarkFileClass;
 typedef struct _KzBookmarkFileType     KzBookmarkFileType;
 
@@ -49,18 +49,19 @@ typedef enum {
 typedef enum {
 	KZ_BOOKMARK_FILE_STATE_NORMAL,
 	KZ_BOOKMARK_FILE_STATE_LOADING,
-	KZ_BOOKMARK_FILE_STATE_SAVING
+	KZ_BOOKMARK_FILE_STATE_SAVING,
+	KZ_BOOKMARK_FILE_STATE_DISPOSING
 } KzBookmarkFileState;
 
 struct _KzBookmarkFile
 {
-	KzBookmark         parent;
+	KzBookmarkFolder    parent;
 	KzBookmarkFileFlag  flags;
 };
 
 struct _KzBookmarkFileClass
 {
-	KzBookmarkClass parent_class;
+	KzBookmarkFolderClass parent_class;
 
 	/* -- signals -- */
 
@@ -68,7 +69,7 @@ struct _KzBookmarkFileClass
 	void (*load_completed)     (KzBookmarkFile  *bookmark_file);
 	void (*save_start)         (KzBookmarkFile  *bookmark_file);
 	void (*save_completed)     (KzBookmarkFile  *bookmark_file);
-	void (*update)             (KzBookmarkFile  *bookmark_file);
+	gboolean (*update)         (KzBookmarkFile  *bookmark_file);
 
 	void (*error)              (KzBookmarkFile  *bookmark_file,
 				    const gchar     *message);
@@ -105,6 +106,8 @@ KzBookmarkFile     *kz_bookmark_file_create_new	(const gchar *location,
 /*
  * "location" means location of bookmark_file file itself.
  */
+const gchar    *kz_bookmark_file_get_document_title
+						(KzBookmarkFile  *bookmark_file);
 const gchar    *kz_bookmark_file_get_location	(KzBookmarkFile  *bookmark_file);
 const gchar    *kz_bookmark_file_get_file_type	(KzBookmarkFile  *bookmark_file);
 const gchar    *kz_bookmark_file_get_xmlrpc	(KzBookmarkFile  *bookmark_file);
@@ -113,6 +116,9 @@ const gchar    *kz_bookmark_file_get_xmlrpc_pass(KzBookmarkFile  *bookmark_file)
 guint		kz_bookmark_file_get_interval   (KzBookmarkFile  *bookmark_file);
 gboolean        kz_bookmark_file_is_editable    (KzBookmarkFile  *bookmark_file);
 
+void		kz_bookmark_file_set_document_title
+						(KzBookmarkFile  *bookmark_file,
+						 const gchar 	 *document_title);
 void		kz_bookmark_file_set_location	(KzBookmarkFile  *bookmark_file,
 						 const gchar     *location);
 void		kz_bookmark_file_set_file_type	(KzBookmarkFile  *bookmark_file,
@@ -152,6 +158,8 @@ void		kz_bookmark_file_xmlrpc_move     (KzBookmarkFile *file,
 						  KzBookmark *sibling,
 						  KzBookmark *child);
 gboolean	kz_bookmark_file_is_update	 (KzBookmarkFile *bookmark_file);
+void            kz_bookmark_file_set_state       (KzBookmarkFile *bookmark_file,
+						  KzBookmarkFileState state);
 
 G_END_DECLS
 
