@@ -88,7 +88,7 @@ kz_print_dialog_class_init (KzPrintDialogClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->constructor  = constructor;
 	gobject_class->dispose      = dispose;
@@ -262,7 +262,8 @@ constructor (GType                  type,
 	     guint                  n_props,
 	     GObjectConstructParam *props)
 {
-	GtkWidget *widget, *embed;
+	GtkWidget *widget;
+        KzEmbed *embed;
 	KzPrintDialog *print;
 	GObject *object;
 	GObjectClass *klass = G_OBJECT_CLASS(kz_print_dialog_parent_class);
@@ -277,13 +278,13 @@ constructor (GType                  type,
 	gtk_window_set_title(GTK_WINDOW(print), _("Print"));
 	gtk_window_set_transient_for(GTK_WINDOW(print), GTK_WINDOW(print->parent_window));
 
-	embed = KZ_WINDOW_CURRENT_PAGE(KZ_WINDOW(print->parent_window));
+	embed = KZ_WINDOW_CURRENT_EMBED(KZ_WINDOW(print->parent_window));
 
-	if (KZ_IS_EMBED(embed))
+	if (embed)
 	{
 		GList *printer_list, *node;
 
-		printer_list = kz_embed_get_printer_list(KZ_EMBED(embed));
+		printer_list = kz_embed_get_printer_list(embed);
 
 		for (node = g_list_first(printer_list); node; node = g_list_next(node))
 		{
