@@ -104,7 +104,7 @@ G_BEGIN_DECLS
 #define KZ_FREEZE_SESSION kz_app_freeze_session(kz_app_get())
 #define KZ_THAW_SESSION kz_app_thaw_session(kz_app_get())
 
-#define KZ_GET_WINDOW_FROM_TAB(tab) kz_app_get_window_from_tab(kz_app_get(), tab)
+#define KZ_GET_WINDOW_FROM_EMBEd(embed) kz_app_get_window_from_embed(kz_app_get(), embed)
 
 G_END_DECLS
 
