@@ -28,6 +28,7 @@
 #include "kz-profile.h"
 #include "kz-root-bookmark.h"
 #include "kz-migemo.h"
+#include "kz-embed.h"
 
 #include "egg-pixbuf-thumbnail.h"
 
@@ -55,7 +56,7 @@ struct _KzAppClass
 	/* -- signals -- */
 	void (*thumbnail_created) (KzApp *app,
 				   const gchar *uri,
-				   EggPixbufThumbSize size);
+				   EggPixbufThumbnailSize size);
 };
 
 GType      kz_app_get_type      (void) G_GNUC_CONST;
@@ -114,7 +115,8 @@ KzRootBookmark *kz_app_get_root_bookmark   (KzApp *app);
 KzProfile      *kz_app_get_profile         (KzApp *app);
 KzProfile      *kz_app_get_proxy           (KzApp *app);
 const GList    *kz_app_get_window_list     (KzApp *app);
-GtkWidget      *kz_app_get_window_from_tab (KzApp *app, GtkWidget *tab);
+GtkWidget      *kz_app_get_window_from_embed
+                                           (KzApp *app, KzEmbed *embed);
 void            kz_app_freeze_session      (KzApp *app);
 void            kz_app_thaw_session        (KzApp *app);
 #if USE_MIGEMO
@@ -125,7 +127,7 @@ gboolean        kz_app_create_thumbnail    (KzApp *app,
 					    GdkPixbuf *pixbuf,
 					    const gchar *uri,
 					    time_t mtime,
-					    EggPixbufThumbSize size);
+					    EggPixbufThumbnailSize size);
 
 G_END_DECLS
 
