@@ -73,11 +73,6 @@ typedef enum {
 	KZ_EMBED_LINK_GUARD
 } KzEmbedNavLink;
 
-typedef enum {
-	KZ_EMBED_COPY_NORMAL,
-	KZ_EMBED_COPY_SOURCE
-} KzEmbedCopyType;
-
 struct _KzEmbedIFace {
 	GTypeInterface g_iface;
 
@@ -86,12 +81,14 @@ struct _KzEmbedIFace {
 						 const gchar   *url);
 	void		(*view_source)		(KzEmbed       *kzembed,
 						 const gchar   *url);
+	void            (*view_current_page_source_in_new)
+                                                (KzEmbed       *kzembed,
+						 KzEmbed       *new_embed);
 
 	gboolean	(*is_loading)		(KzEmbed       *kzembed);
 
 	const gchar    *(*get_title)		(KzEmbed       *kzembed);
 	const gchar    *(*get_location)		(KzEmbed       *kzembed);
-	gchar	       *(*ensure_title)		(KzEmbed       *kzembed);
 	gchar	       *(*get_link_message)	(KzEmbed       *kzembed);
 
 	gdouble		(*get_progress)		(KzEmbed       *kzembed);
@@ -107,10 +104,10 @@ struct _KzEmbedIFace {
 	gchar	       *(*get_selection_string)	(KzEmbed       *kzembed);
 
 	gboolean	(*find)			(KzEmbed       *kzembed,
-						 const char    *keyword,
+						 const gchar   *keyword,
 						 gboolean	backward);
 	gboolean	(*incremental_search)	(KzEmbed       *kzembed,
-						 const char    *keyword,
+						 const gchar   *keyword,
 						 gboolean	backward);
 
 	gboolean	(*selection_is_collapsed)
@@ -123,23 +120,17 @@ struct _KzEmbedIFace {
 	gboolean	(*get_dest_anchors)	(KzEmbed       *kzembed,
 						 GList	      **list);
 
-	void		(*copy_page)		(KzEmbed       *kzembed,
-						 KzEmbed       *dkzembed,
-						 KzEmbedCopyType type);
+	void		(*copy_page)		(KzEmbed       *src_embed,
+						 KzEmbed       *dest_embed);
 
-	gboolean	(*shistory_copy)	(KzEmbed       *source,
-						 KzEmbed       *dest,
-						 gboolean	back_history,
-						 gboolean	forward_history,
-						 gboolean	set_current);
 	gboolean	(*shistory_get_pos)	(KzEmbed       *kzembed,
 						 gint	       *pos,
 						 gint	       *count);
 	void		(*shistory_get_nth)	(KzEmbed       *kzembed, 
 						 gint		nth,
 						 gboolean	is_relative,
-						 gchar	      **aUrl,
-						 gchar	      **aTitle);
+						 gchar	      **uri,
+						 gchar	      **title);
 
 	void		(*reload)		(KzEmbed       *kzembed,
 						 KzEmbedReloadFlag flags);
@@ -177,9 +168,9 @@ struct _KzEmbedIFace {
 						 gint		index);
 
 	void		(*do_command)		(KzEmbed       *kzembed,
-						 const char    *command);
+						 const gchar   *command);
 	gboolean	(*can_do_command)	(KzEmbed       *kzembed,
-						 const char    *command);
+						 const gchar   *command);
 
 	gboolean	(*get_lock)		(KzEmbed       *kzembed);
 	void		(*set_lock)		(KzEmbed       *kzembed,
@@ -190,19 +181,19 @@ struct _KzEmbedIFace {
 	gchar	       *(*get_selection_source)	(KzEmbed       *kzembed);
 #endif
 	void		(*set_encoding)		(KzEmbed       *kzembed,
-						 const char    *encoding);
+						 const gchar   *encoding);
 	void		(*get_encoding)		(KzEmbed       *kzembed,
-						 char	      **encoding,
+						 gchar	      **encoding,
 						 gboolean      *forced);
 	void		(*print)		(KzEmbed       *kzembed);
 	void		(*print_preview)	(KzEmbed       *kzembed);
 	GList	       *(*get_printer_list)	(KzEmbed       *kzembed);
 	void		(*create_thumbnail)	(KzEmbed       *kzembed,
-						 EggPixbufThumbSize size);
+						 EggPixbufThumbnailSize size);
 
 
 	gboolean	(*save_with_content)	(KzEmbed       *kzembed,
-						 const char    *rawfilename);
+						 const gchar   *rawfilename);
 
 	gboolean	(*set_text_into_textarea)
 						(KzEmbed       *kzembed,
@@ -227,9 +218,9 @@ struct _KzEmbedIFace {
 						 const gchar   *storedir);
 
 	void		(*set_history)		(KzEmbed       *kzembed,
-						 KzBookmark    *history);
+						 KzBookmarkFolder *history);
 	void		(*get_history)		(KzEmbed       *kzembed,
-						 KzBookmark    *history);
+						 KzBookmarkFolder *history);
 	guint		(*get_last_modified)	(KzEmbed       *kzembed);
 
 #if 0
@@ -237,8 +228,8 @@ struct _KzEmbedIFace {
 	void		(*set_view_mode)	(KzEmbed       *kzembed);
 #endif
 	void            (*fine_scroll)          (KzEmbed       *embed,
-						 int            horiz,
-						 int            vert);
+						 gint           horiz,
+						 gint           vert);
 	void		(*page_down)		(KzEmbed       *kzembed);
 	void		(*page_up)		(KzEmbed       *kzembed);
 	
@@ -252,17 +243,21 @@ struct _KzEmbedIFace {
         const gchar    *(*get_name)             (KzEmbed       *kzembed);
 
 	/* signals */
-	void     (*link_message)        (KzEmbed      *embed);
+	void     (*link_message)        (KzEmbed      *embed,
+                                         const gchar  *message);
 	void     (*js_status)           (KzEmbed      *embed);
-	void     (*location)            (KzEmbed      *embed);
-	void     (*title)               (KzEmbed      *embed);
-	void     (*progress)            (KzEmbed      *embed);
+	void     (*location)            (KzEmbed      *embed,
+                                         const gchar  *location);
+	void     (*title)               (KzEmbed      *embed,
+                                         const gchar  *title);
+	void     (*progress)            (KzEmbed      *embed,
+                                         gdouble      ratio);
 	void     (*net_start)           (KzEmbed      *embed);
 	void     (*net_stop)            (KzEmbed      *embed);
 	void     (*new_window)          (KzEmbed      *embed,
 					 KzEmbed     **new_embed);
 	gint     (*open_uri)            (KzEmbed      *embed,
-					 const char   *uri);
+					 const gchar  *uri);
 	void     (*size_to)             (KzEmbed      *embed,
 					 gint          width,
 					 gint          height);
@@ -309,6 +304,9 @@ void		kz_embed_load_url		(KzEmbed       *kzembed,
 						 const gchar   *url);
 void		kz_embed_view_source		(KzEmbed       *kzembed,
 						 const gchar   *url);
+void            kz_embed_view_current_page_source_in_new
+                                                (KzEmbed       *kzembed,
+                                                 KzEmbed       *new_embed);
 
 gboolean	kz_embed_is_loading		(KzEmbed       *kzembed);
 
@@ -331,10 +329,10 @@ gchar	       *kz_embed_get_selection_string
 						(KzEmbed       *kzembed);
 
 gboolean	kz_embed_find			(KzEmbed       *kzembed,
-						 const char    *keyword,
+						 const gchar   *keyword,
 						 gboolean	backward);
 gboolean	kz_embed_incremental_search	(KzEmbed       *kzembed,
-						 const char    *keyword,
+						 const gchar   *keyword,
 						 gboolean	backward);
 
 gboolean	kz_embed_selection_is_collapsed
@@ -347,15 +345,9 @@ gboolean	kz_embed_get_links		(KzEmbed       *kzembed,
 gboolean	kz_embed_get_dest_anchors	(KzEmbed       *kzembed,
 						 GList	      **list);
 
-void		kz_embed_copy_page		(KzEmbed       *kzembed,
-						 KzEmbed       *dkzembed,
-						 KzEmbedCopyType type);
+void		kz_embed_copy_page		(KzEmbed       *src_embed,
+						 KzEmbed       *dest_embed);
 
-gboolean	kz_embed_shistory_copy		(KzEmbed       *source,
-						 KzEmbed       *dest,
-						 gboolean	back_history,
-						 gboolean	forward_history,
-						 gboolean	set_current);
 gboolean	kz_embed_shistory_get_pos	(KzEmbed       *kzembed,
 						 gint	       *pos,
 						 gint	       *count);
@@ -401,9 +393,9 @@ void		kz_embed_go_history_index	(KzEmbed       *kzembed,
 						 gint		index);
 
 void		kz_embed_do_command		(KzEmbed       *kzembed,
-						 const char    *command);
+						 const gchar   *command);
 gboolean	kz_embed_can_do_command		(KzEmbed       *kzembed,
-						 const char    *command);
+						 const gchar   *command);
 
 gboolean	kz_embed_get_lock		(KzEmbed       *kzembed);
 void		kz_embed_set_lock		(KzEmbed       *kzembed,
@@ -412,15 +404,15 @@ void		kz_embed_set_lock		(KzEmbed       *kzembed,
 gchar	       *kz_embed_get_body_text		(KzEmbed       *kzembed);
 
 void		kz_embed_set_encoding		(KzEmbed       *kzembed,
-						 const char    *encoding);
+						 const gchar   *encoding);
 void		kz_embed_get_encoding		(KzEmbed       *kzembed,
-						 char	      **encoding,
+						 gchar	      **encoding,
 						 gboolean      *forced);
 void		kz_embed_print			(KzEmbed       *kzembed);
 void		kz_embed_print_preview		(KzEmbed       *kzembed);
 GList	       *kz_embed_get_printer_list	(KzEmbed       *kzembed);
 void		kz_embed_create_thumbnail	(KzEmbed       *kzembed,
-						 EggPixbufThumbSize size);
+						 EggPixbufThumbnailSize size);
 
 
 gboolean	kz_embed_save_with_content	(KzEmbed       *kzembed,
@@ -447,15 +439,15 @@ gchar	       *kz_embed_get_html_with_contents
 						 const gchar   *storedir);
 
 void		kz_embed_set_history		(KzEmbed       *kzembed,
-						 KzBookmark    *history);
+						 KzBookmarkFolder *history);
 void		kz_embed_get_history		(KzEmbed       *kzembed,
-						 KzBookmark    *history);
+						 KzBookmarkFolder *history);
 
 guint		kz_embed_get_last_modified	(KzEmbed       *kzembed);	
 
 void            kz_embed_fine_scroll            (KzEmbed       *kzembed,
-						 int            horiz,
-						 int            vert);
+						 gint           horiz,
+						 gint           vert);
 void		kz_embed_page_up		(KzEmbed       *kzembed);
 void		kz_embed_page_down		(KzEmbed       *kzembed);
 
