@@ -3,6 +3,7 @@
 
 #include "kz-enum-types.h"
 #include "kazehakase.h"
+#include "kz-objects.h"
 #include "kz-app.h"
 #include "kz-module.h"
 #include "kz-module-impl.h"
@@ -128,20 +129,6 @@ kz_embed_nav_link_get_type (void)
   }
   return etype;
 }
-GType
-kz_embed_copy_type_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { KZ_EMBED_COPY_NORMAL, "KZ_EMBED_COPY_NORMAL", "normal" },
-      { KZ_EMBED_COPY_SOURCE, "KZ_EMBED_COPY_SOURCE", "source" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("KzEmbedCopyType", values);
-  }
-  return etype;
-}
 
 /* enumerations from "kz-embed-event.h" */
 GType
