@@ -51,8 +51,11 @@ struct _KzTabLabelPrivate
 	gboolean lock;
 	gboolean auto_refresh;
 	gboolean javascript;
+	gboolean show_favicon;
+	gboolean create_thumbnail;
 	guint auto_refresh_id;
-	gulong favicon_signal;
+	gint icon_width;
+	gint icon_height;
 };
 #define KZ_TAB_LABEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_TAB_LABEL, KzTabLabelPrivate))
 
@@ -103,13 +106,18 @@ static void     cb_profile_changed              (KzProfile *profile,
 						 const gchar *old_value,
 						 KzTabLabel *kztab);
 
+static void     cb_global_profile_changed       (KzProfile *profile,
+						 const gchar *section,
+						 const gchar *key,
+						 const gchar *old_value,
+						 KzTabLabel *kztab);
+
 /* callbacks for embed */
-static void cb_title_changed (KzEmbed *embed, KzTabLabel *kztab);
-static void cb_location_changed (KzEmbed *embed, KzTabLabel *kztab);
+static void cb_title_changed (KzEmbed *embed, const gchar *title, KzTabLabel *kztab);
+static void cb_location_changed (KzEmbed *embed, const gchar *location, KzTabLabel *kztab);
 static void cb_net_start     (KzEmbed *embed, KzTabLabel *kztab);
 static void cb_net_stop      (KzEmbed *embed, KzTabLabel *kztab);
-static void cb_progress      (KzEmbed *embed, KzTabLabel *kztab);
-static void cb_destroy       (GtkObject *object,  KzTabLabel *kztab);
+static void cb_progress      (KzEmbed *embed, gdouble ratio, KzTabLabel *kztab);
 
 static void make_progress_circle (GtkWidget *widget);
 
@@ -139,8 +147,8 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	gobject_class = (GObjectClass *) klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	widget_class  = GTK_WIDGET_CLASS(klass);
 
 	/* GtkObject signals */
 	gobject_class->constructor  = constructor;
@@ -174,7 +182,7 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 				      _("The KzEmbed object to observe"),
 				      KZ_TYPE_EMBED,
 				      G_PARAM_READWRITE |
-				      G_PARAM_CONSTRUCT_ONLY));
+				      G_PARAM_CONSTRUCT));
 	g_type_class_add_private (gobject_class, sizeof(KzTabLabelPrivate));
 }
 
@@ -184,7 +192,6 @@ kz_tab_label_init (KzTabLabel *kztab)
 	GtkWidget *close_image, *hbox;
 	GtkRcStyle *style;
 	GtkRequisition size;
-	gint width, height;
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
 	kztab->kz           = NULL;
@@ -205,14 +212,17 @@ kz_tab_label_init (KzTabLabel *kztab)
 	priv->lock              = FALSE;
 	priv->auto_refresh      = FALSE;
 	priv->auto_refresh_id   = 0;
-	priv->favicon_signal    = 0;
 	
-	kztab->favicon_pixbuf   = NULL;
+	priv->show_favicon      = TRUE;
+	priv->create_thumbnail  = FALSE;
+
+	KZ_CONF_GET("Tab", "show_favicon", priv->show_favicon, BOOL);
+	KZ_CONF_GET("Global", "create_thumbnail", priv->create_thumbnail, BOOL);
+
 	kztab->circle           = NULL;
 	kztab->mask             = NULL;
 
-	kztab->history    = kz_bookmark_pure_folder_new();
-	kz_bookmark_set_title(kztab->history, _("Tab"));
+	kztab->history    = KZ_BOOKMARK_FOLDER(kz_bookmark_folder_new(_("Tab")));
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(kztab->eventbox), hbox);
@@ -230,8 +240,8 @@ kz_tab_label_init (KzTabLabel *kztab)
 	gtk_widget_show (kztab->label);
 
 	/* favicon */
-	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-	gtk_widget_set_size_request(kztab->favicon, width, height);
+	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &priv->icon_width, &priv->icon_height);
+	gtk_widget_set_size_request(kztab->favicon, priv->icon_width, priv->icon_height);
 
 	/* close button */
 	gtk_button_set_relief(GTK_BUTTON(kztab->close_button), GTK_RELIEF_NONE);
@@ -274,10 +284,41 @@ kz_tab_label_init (KzTabLabel *kztab)
 
 	g_signal_connect(KZ_GET_GLOBAL_PROFILE, "changed::Tab",
 			 G_CALLBACK(cb_profile_changed), kztab);
+	g_signal_connect(KZ_GET_GLOBAL_PROFILE, "changed::Global",
+			 G_CALLBACK(cb_global_profile_changed), kztab);
 	kz_tab_label_sync_to_profile(kztab);
 	kz_tab_label_set_text(kztab, NULL);
 }
 
+static void
+connect_embed_signals (KzTabLabel *kztab)
+{
+	g_signal_connect(kztab->kzembed, "kz-title",
+			 G_CALLBACK(cb_title_changed), kztab);
+	g_signal_connect(kztab->kzembed, "kz-location",
+			 G_CALLBACK(cb_location_changed), kztab);
+	g_signal_connect(kztab->kzembed, "kz-net-start",
+			 G_CALLBACK(cb_net_start), kztab);
+	g_signal_connect(kztab->kzembed, "kz-net-stop",
+			 G_CALLBACK(cb_net_stop), kztab);
+	g_signal_connect(kztab->kzembed, "kz-progress",
+			 G_CALLBACK(cb_progress), kztab);
+}
+
+static void
+disconnect_embed_signals (KzTabLabel *kztab)
+{
+	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+					     cb_net_start, kztab);
+	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+					     cb_net_stop, kztab);
+	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+					     cb_title_changed, kztab);
+	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+					     cb_location_changed, kztab);
+	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+					     cb_progress, kztab);
+}
 
 static GObject*
 constructor (GType                  type,
@@ -293,19 +334,7 @@ constructor (GType                  type,
 
 	kztab = KZ_TAB_LABEL(object);
 
-	/* set signals */
-	g_signal_connect(kztab->kzembed, "kz-title",
-			 G_CALLBACK(cb_title_changed), kztab);
-	g_signal_connect(kztab->kzembed, "kz-location",
-			 G_CALLBACK(cb_location_changed), kztab);
-	g_signal_connect(kztab->kzembed, "kz-net-start",
-			 G_CALLBACK(cb_net_start), kztab);
-	g_signal_connect(kztab->kzembed, "kz-net-stop",
-			 G_CALLBACK(cb_net_stop), kztab);
-	g_signal_connect(kztab->kzembed, "kz-progress",
-			 G_CALLBACK(cb_progress), kztab);
-	g_signal_connect(kztab->kzembed, "destroy",
-			 G_CALLBACK(cb_destroy), kztab);
+	connect_embed_signals(kztab);
 
 	/* set label text */
 	title = kz_embed_ensure_title(kztab->kzembed);
@@ -328,38 +357,16 @@ dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func(KZ_GET_GLOBAL_PROFILE,
 					     G_CALLBACK(cb_profile_changed),
 					     kztab);
+	g_signal_handlers_disconnect_by_func(KZ_GET_GLOBAL_PROFILE,
+					     G_CALLBACK(cb_global_profile_changed),
+					     kztab);
 	if (kztab->kzembed)
-	{
-	
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_net_start, kztab);
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_net_stop, kztab);
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_title_changed, kztab);
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_location_changed, kztab);
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_progress, kztab);
-		g_signal_handlers_disconnect_by_func(kztab->kzembed,
-						     cb_destroy, kztab);
-		g_object_unref(kztab->kzembed);
-		kztab->kzembed = NULL;
-	}
-
-
-	/* if connected signal exists, disconnect it. */
-	if (priv->favicon_signal)
-	{
-		g_signal_handler_disconnect(KZ_GET_FAVICON,
-					    priv->favicon_signal);
-		priv->favicon_signal = 0;
-	}
+		kz_tab_label_set_embed(kztab, NULL);
 
 	if (kztab->history)
 	{
-		kz_bookmark_set_javascript(kztab->history, priv->javascript);
-		kz_bookmark_remove(kztab->kz->tabs, kztab->history);
+		kz_bookmark_folder_set_javascript(kztab->history, priv->javascript);
+		kz_bookmark_folder_remove(KZ_BOOKMARK_FOLDER(kztab->kz->tabs), KZ_BOOKMARK(kztab->history));
 		kz_window_append_closed_tab(kztab->kz, kztab->history);
 		g_object_unref(kztab->history);
 		kztab->history = NULL;
@@ -371,11 +378,6 @@ dispose (GObject *object)
 		kztab->kz = NULL;
 	}
 
-	if (kztab->favicon_pixbuf)
-	{
-		g_object_unref(kztab->favicon_pixbuf);
-		kztab->favicon_pixbuf = NULL;
-	}
 	if (kztab->circle)
 	{
 		g_object_unref(kztab->circle);
@@ -406,7 +408,7 @@ set_property (GObject         *object,
 		tablabel->kz = g_object_ref(g_value_get_object(value));
 		break;
 	case PROP_KZ_EMBED:
-		tablabel->kzembed = g_object_ref(g_value_get_object(value));
+		kz_tab_label_set_embed(tablabel, g_value_get_object(value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -479,18 +481,13 @@ idle_create_thumbnail (gpointer data)
 {
 	KzTabLabel *kztab;
 	KzEmbed *embed;
-	EggPixbufThumbSize size = EGG_PIXBUF_THUMB_LARGE;
-	gboolean create_thumbnail = FALSE;
+	EggPixbufThumbnailSize size = EGG_PIXBUF_THUMBNAIL_LARGE;
 	const gchar *uri;
 	guint last_modified;
 
 	kztab = data;
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(kztab), FALSE);
 
-	KZ_CONF_GET("Global", "create_thumbnail", create_thumbnail, BOOL);
-	if (!create_thumbnail)
-		return FALSE;
-
 	embed = kztab->kzembed;
 	uri = kz_embed_get_location(embed);
 	if (!uri)
@@ -518,21 +515,26 @@ kz_tab_label_set_state (KzTabLabel *kztab,
 			KzTabLabelState state)
 {
 	gchar *title;
+	KzTabLabelPrivate *priv;
 	
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	if (!GTK_WIDGET_REALIZED(kztab)) return;
 
-	if (kztab->state != KZ_TAB_LABEL_STATE_NORMAL &&
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
+	if (priv->create_thumbnail &&
+	    kztab->state != KZ_TAB_LABEL_STATE_NORMAL &&
 	    state == KZ_TAB_LABEL_STATE_NORMAL)
+	{
 		g_idle_add(idle_create_thumbnail, kztab);
+	}
 
 	kztab->state = state;
 	if (state == KZ_TAB_LABEL_STATE_NORMAL)
 	{
 		kz_tab_label_set_visited(kztab);
 	}
-	
+
 	title = kz_embed_ensure_title(kztab->kzembed);
 	kz_tab_label_set_text(kztab, title);
 	g_free(title);
@@ -580,16 +582,6 @@ kz_tab_label_set_show_lock_button (KzTabLabel *kztab, gboolean show)
         g_object_set(kztab->lock_button, "visible", show, NULL);
 }
 
-
-void
-kz_tab_label_set_show_favicon (KzTabLabel *kztab, gboolean show)
-{
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-
-        g_object_set(kztab->favicon, "visible", show && kztab->favicon_pixbuf, NULL);
-}
-
-
 static void
 realize (GtkWidget *widget)
 {
@@ -630,7 +622,7 @@ button_press (GtkWidget *widget, GdkEventButton *event)
 #endif
 	if (event->button == 2)
 	{
-		kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+		kz_window_close_tab(kztab->kz, kztab->kzembed);
 	}
 	if (event->type == GDK_2BUTTON_PRESS)
 	{
@@ -733,7 +725,6 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 	gint width = 80;
 	gboolean fix = TRUE;
 	gboolean show_close = TRUE;
-	gboolean show_favicon = TRUE;
 	gchar *color;
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
@@ -753,10 +744,6 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 	/* lock button */
 	kz_tab_label_set_show_lock_button(kztab, priv->lock);
 
-	/* favicon */
-	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
-	kz_tab_label_set_show_favicon(kztab, show_favicon);
-
 	/* label color */
 	color = KZ_CONF_GET_STR("Tab" , "normal_color");
 	if (!color)
@@ -788,27 +775,26 @@ static void
 make_progress_circle(GtkWidget *widget)
 {
 	GdkGC *gc;
-	gint width, height;
 	GdkColormap *colormap;
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
 
-	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 	kztab->circle = gdk_pixmap_new(widget->window,
-				       width, height, -1);
+				       priv->icon_width, priv->icon_height, -1);
 	kztab->mask = gdk_pixmap_new(widget->window,
-				     width, height, 1);
+				     priv->icon_width, priv->icon_height, 1);
 	gc = gdk_gc_new(kztab->mask);
 	gdk_draw_rectangle(kztab->mask,
 			   gc,
 			   TRUE,
 			   0, 0,
-			   width, height);
+			   priv->icon_width, priv->icon_height);
 	gdk_gc_set_function(gc, GDK_INVERT);
 	gdk_draw_arc(kztab->mask,
 		     gc,
 		     TRUE,
 		     0, 0,
-		     width, height,
+		     priv->icon_width, priv->icon_height,
 		     90 * 64, 360 * 64);
 	g_object_unref(gc);
 
@@ -820,12 +806,12 @@ make_progress_circle(GtkWidget *widget)
 			   widget->style->white_gc, 
 			   TRUE,
 			   0, 0,
-			   width, height);
+			   priv->icon_width, priv->icon_height);
 	gdk_draw_arc(kztab->circle,
 		     gc, 
 		     TRUE,
 		     0, 0,
-		     width, height,
+		     priv->icon_width, priv->icon_height,
 		     90 * 64, 360 * 64);
 	g_object_unref(colormap);
 	g_object_unref(gc);
@@ -836,7 +822,7 @@ static void
 cb_close_button_clicked (GtkWidget *button, KzTabLabel *kztab)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+	kz_window_close_tab(kztab->kz, kztab->kzembed);
 }
 
 static void
@@ -848,39 +834,54 @@ cb_profile_changed (KzProfile *profile,
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	kz_tab_label_sync_to_profile (kztab);
+	if (key && strcmp(key, "show_favicon"))
+	{
+		KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
+		KZ_CONF_GET("Tab", "show_favicon", priv->show_favicon, BOOL);
+		if (priv->show_favicon)
+			gtk_widget_show(kztab->favicon);
+		else
+			gtk_widget_hide(kztab->favicon);
+	}
+	kz_tab_label_sync_to_profile(kztab);
+}
+
+static void
+cb_global_profile_changed (KzProfile *profile,
+			   const gchar *section,
+			   const gchar *key,
+			   const gchar *old_value,
+			   KzTabLabel *kztab)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	if (key && strcmp(key, "create_thumbnail"))
+	{
+		KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
+		KZ_CONF_GET("Global", "create_thumbnail", priv->create_thumbnail, BOOL);
+	}
 }
 
 /* callbacks for embed */
 
 static void
-cb_title_changed (KzEmbed *embed, KzTabLabel *kztab)
+cb_title_changed (KzEmbed *embed, const gchar *title, KzTabLabel *kztab)
 {
-	gchar *title;
-
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	title = kz_embed_ensure_title(kztab->kzembed);
 	kz_tab_label_set_text(kztab, title);
-	g_free(title);
 }
 
 static void
-cb_location_changed (KzEmbed *embed, KzTabLabel *kztab)
+cb_location_changed (KzEmbed *embed, const gchar *location, KzTabLabel *kztab)
 {
-	KzBookmark *bookmark = NULL;
-	guint cur;
-	GList *children;
+	KzBookmark *bookmark;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	cur = kz_bookmark_get_current(kztab->history);
-	children = kz_bookmark_get_children(kztab->history);
-	bookmark = KZ_BOOKMARK(g_list_nth_data(children, cur));
-	g_list_free(children);
-
+	bookmark = kz_bookmark_folder_get_current_bookmark(kztab->history);
 	if (bookmark)
-		kz_bookmark_set_link(bookmark, kz_embed_get_location(embed));
+		kz_bookmark_set_link(bookmark, location);
 }
 
 static void
@@ -890,12 +891,8 @@ cb_net_start (KzEmbed *embed, KzTabLabel *kztab)
 
 	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADING);
 
-	if (kztab->favicon_pixbuf)
-	{
-		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon), NULL);
-		g_object_unref(kztab->favicon_pixbuf);
-		kztab->favicon_pixbuf = NULL;
-	}
+	gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon), NULL);
+
 	/* show favicon widget for progress maru. */
 	gtk_widget_show(kztab->favicon);
 }
@@ -904,7 +901,7 @@ static void
 cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 {
 	KzNotebook *note;
-	gboolean show_favicon;
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
@@ -919,15 +916,10 @@ cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 	}
 	else
 	{
-		KzBookmark *bookmark = NULL;
-		guint cur, last_modified, last_visited = 0;
-		GList *children;
+		KzBookmark *bookmark;
+		guint last_modified, last_visited = 0;
 	
-		cur = kz_bookmark_get_current(kztab->history);
-		children = kz_bookmark_get_children(kztab->history);
-		bookmark = KZ_BOOKMARK(g_list_nth_data(children, cur));
-		g_list_free(children);
-
+		bookmark = kz_bookmark_folder_get_current_bookmark(kztab->history);
 		if (bookmark)
 			last_visited = kz_bookmark_get_last_visited(bookmark);
 		last_modified = kz_embed_get_last_modified(embed);
@@ -938,71 +930,59 @@ cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 			kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADED);
 	}
 
-	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
-
-	if (show_favicon)
+	if (priv->show_favicon)
 	{
 		const gchar *uri;
+		GdkPixbuf *favicon_pixbuf;
+
 		uri = kz_embed_get_location(KZ_EMBED(kztab->kzembed));
-		kztab->favicon_pixbuf = kz_favicon_get_pixbuf(KZ_GET_FAVICON, uri,
-							      GTK_ICON_SIZE_MENU);
+		favicon_pixbuf = kz_favicon_get_pixbuf(KZ_GET_FAVICON, uri,
+						       GTK_ICON_SIZE_MENU);
 		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
-					  kztab->favicon_pixbuf);
+					  favicon_pixbuf);
+		if (favicon_pixbuf)
+			g_object_unref(favicon_pixbuf);
 	}
-
-	kz_tab_label_set_show_favicon(kztab, show_favicon);
 }
 
 
 
 static void
-cb_destroy (GtkObject *object, KzTabLabel *kztab)
+cb_progress (KzEmbed *embed, gdouble ratio, KzTabLabel *kztab)
 {
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	g_object_unref(kztab->kzembed);
-	kztab->kzembed = NULL;
-}
-
-
-static void
-cb_progress (KzEmbed *embed, KzTabLabel *kztab)
-{
-	gdouble progress;
 	GtkWidget *widget;	       
 	GdkGC *gc;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	widget = GTK_WIDGET(kztab);
-	progress = kz_embed_get_progress(KZ_EMBED(embed));
 
 	if(GTK_WIDGET_REALIZED(widget))
 	{
 		GdkBitmap *mask;
-		gint width, height;
-		gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+		KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
 
 		mask = gdk_pixmap_new(widget->window,
-				      width, height, 1);
+				      priv->icon_width, priv->icon_height, 1);
 		gc = gdk_gc_new(mask);
 		gdk_draw_rectangle(mask,
 				   gc,
 				   TRUE,
 				   0, 0,
-				   width, height);
+				   priv->icon_width, priv->icon_height);
 		gdk_gc_set_function(gc, GDK_INVERT);
 		gdk_draw_rectangle(mask,
 				   gc,
 				   TRUE,
 				   0, 0,
-				   width, height);
+				   priv->icon_width, priv->icon_height);
 		gdk_gc_set_function(gc, GDK_COPY);
 		gdk_draw_arc(mask,
 			     gc,
 			     TRUE,
 			     0, 0,
-			     width, height,
-			     90 * 64, 360 * 64 * progress);
+			     priv->icon_width, priv->icon_height,
+			     90 * 64, 360 * 64 * ratio);
 		gdk_gc_set_function(gc, GDK_AND);
 		gdk_draw_drawable(mask,
 				  gc,
@@ -1041,7 +1021,7 @@ kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 	kz_embed_set_lock(kztab->kzembed, lock);
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 	
-	kz_bookmark_set_lock(kztab->history, lock);
+	kz_bookmark_folder_set_lock(kztab->history, lock);
 	kz_tab_label_sync_to_profile(kztab);
 }
 
@@ -1067,7 +1047,7 @@ kz_tab_label_set_javascript (KzTabLabel *kztab, gboolean javascript)
 	kz_embed_set_allow_javascript(kztab->kzembed, javascript);
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 	
-	kz_bookmark_set_javascript(kztab->history, javascript);
+	kz_bookmark_folder_set_javascript(kztab->history, javascript);
 	kz_tab_label_sync_to_profile(kztab);
 }
 
@@ -1111,7 +1091,7 @@ kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
-	if(auto_refresh)
+	if (auto_refresh)
 	{
 		gint auto_refresh_interval_sec = DEFAULT_AUTO_REFRESH_INTERVAL_SEC;
 		KZ_CONF_GET("Tab", "auto_refresh_interval_sec", auto_refresh_interval_sec, INT);
@@ -1134,12 +1114,12 @@ kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 	priv->auto_refresh = auto_refresh;
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 
-	kz_bookmark_set_auto_refresh(kztab->history, auto_refresh);
+	kz_bookmark_folder_set_auto_refresh(kztab->history, auto_refresh);
 	kz_tab_label_sync_to_profile(kztab);
 }
 
 void
-kz_tab_label_set_history (KzTabLabel *kztab, KzBookmark *history)
+kz_tab_label_set_history (KzTabLabel *kztab, KzBookmarkFolder *history)
 {	
         gboolean allow_javacript;
 
@@ -1148,48 +1128,56 @@ kz_tab_label_set_history (KzTabLabel *kztab, KzBookmark *history)
 	g_object_unref(kztab->history);
 	kztab->history = g_object_ref(history);
 
-	if (kz_bookmark_get_lock(history))
-	{
+	if (kz_bookmark_folder_get_lock(history))
 		kz_tab_label_set_lock(kztab, TRUE);
-	}
-	if (kz_bookmark_get_auto_refresh(history))
-	{
+
+	if (kz_bookmark_folder_get_auto_refresh(history))
 		kz_tab_label_set_auto_refresh(kztab, TRUE);
-	}
         
-        allow_javacript = kz_bookmark_get_javascript(history);
+        allow_javacript = kz_bookmark_folder_get_javascript(history);
         kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), allow_javacript);
         kz_tab_label_set_javascript(kztab, allow_javacript);
-	
+
 	kz_embed_set_history(kztab->kzembed, kztab->history);
 }
 static void
 kz_tab_label_set_visited (KzTabLabel *kztab)
 {
-	KzBookmark *bookmark = NULL;
+	KzBookmark *bookmark;
 	const gchar *uri;
 	GTimeVal now;
-	guint cur;
-	GList *children;
 	
 	g_get_current_time(&now);
 
-	cur = kz_bookmark_get_current(kztab->history);
-	children = kz_bookmark_get_children(kztab->history);
-	bookmark = KZ_BOOKMARK(g_list_nth_data(children, cur));
-	g_list_free(children);
-
+	bookmark = kz_bookmark_folder_get_current_bookmark(kztab->history);
 	if (bookmark)
 		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
 
 	/* find the bookmark which has the same uri from the system bookmark */
 	uri = kz_embed_get_location(kztab->kzembed);
-	bookmark = kz_bookmark_find_bookmark_from_uri(KZ_GET_MENU_BOOKMARK,
-						      uri);
+	bookmark = kz_bookmark_folder_find_bookmark_from_uri(KZ_BOOKMARK_FOLDER(KZ_GET_MENU_BOOKMARK),
+						      	     uri);
 	if (bookmark)
 		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
 }
 
+void
+kz_tab_label_set_embed (KzTabLabel *kztab, KzEmbed *kzembed)
+{
+	if (kztab->kzembed)
+	{
+		disconnect_embed_signals(kztab);
+		g_object_unref(kztab->kzembed);
+		kztab->kzembed = NULL;
+	}
+
+	if (kzembed)
+	{
+		kztab->kzembed = g_object_ref(kzembed);
+		connect_embed_signals(kztab);
+	}
+}
+
 /*
 vim:ts=8:nowrap:ai
 */
