@@ -23,9 +23,9 @@
 
 #include <gtk/gtk.h>
 
+#include "kz-objects.h"
 #include "kz-embed.h"
-#include "kz-window.h"
-#include "kz-bookmark.h"
+#include "kz-bookmark-folder.h"
 
 G_BEGIN_DECLS
 
@@ -36,7 +36,6 @@ G_BEGIN_DECLS
 #define KZ_IS_TAB_LABEL_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), KZ_TYPE_TAB_LABEL))
 #define KZ_TAB_LABEL_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_TAB_LABEL, KzTabLabelClass))
 
-typedef struct _KzTabLabel           KzTabLabel;
 typedef struct _KzTabLabelClass      KzTabLabelClass;
 
 typedef enum
@@ -47,7 +46,6 @@ typedef enum
 	KZ_TAB_LABEL_N_STATE
 } KzTabLabelState;
 
-
 struct _KzTabLabel
 {
 	GtkHBox parent;
@@ -63,10 +61,9 @@ struct _KzTabLabel
 	GtkWidget       *close_button;
 	GtkWidget       *lock_button;
 	GdkBitmap       *mask;
-	GdkPixbuf       *favicon_pixbuf;
 	GdkPixmap       *circle;
 
-	KzBookmark	*history;
+	KzBookmarkFolder *history;
 };
 
 struct _KzTabLabelClass
@@ -92,8 +89,6 @@ void             kz_tab_label_set_show_close_button
 void             kz_tab_label_set_show_lock_button
 						(KzTabLabel *kztab,
 						 gboolean    show);
-void             kz_tab_label_set_show_favicon  (KzTabLabel *kztab,
-						 gboolean    show);
 
 gboolean         kz_tab_label_get_lock          (KzTabLabel *kztab);
 void             kz_tab_label_set_lock          (KzTabLabel *kztab,
@@ -106,7 +101,9 @@ void             kz_tab_label_set_javascript    (KzTabLabel *kztab,
 						 gboolean    javascript);
 
 void		 kz_tab_label_set_history       (KzTabLabel *kztab,
-						 KzBookmark *history);
+						 KzBookmarkFolder *history);
+void		 kz_tab_label_set_embed		(KzTabLabel *kztab,
+						 KzEmbed    *kzembed);
 G_END_DECLS
 
 #endif /* __KZ_TAB_LABEL_H__ */
