@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-sidebar.c 2863 2007-03-14 06:43:16Z ikezoe $
+ *  $Id: kz-bookmarks-sidebar.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-bookmarks-sidebar.h"
@@ -26,6 +26,7 @@
 #include "kazehakase.h"
 #include "kz-actions.h"
 #include "kz-bookmarks-view.h"
+#include "kz-bookmark-separator.h"
 
 
 enum {
@@ -78,7 +79,7 @@ kz_bookmarks_sidebar_class_init (KzBookmarksSidebarClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose      = dispose;
 	gobject_class->set_property = set_property;
