@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-downloader-sidebar.c 3228 2007-07-28 14:50:23Z kous $
+ *  $Id: kz-downloader-sidebar.c 3717 2009-02-16 07:02:02Z ikezoe $
  */
 
 #include "kz-downloader-sidebar.h"
@@ -81,7 +81,7 @@ kz_downloader_sidebar_class_init (KzDownloaderSidebarClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose      = dispose;
 	gobject_class->set_property = set_property;
@@ -117,8 +117,8 @@ update_downloader_list (KzDownloaderSidebar *sidebar)
 
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter,
-				   COLUMN_URI,        downloader->uri,
-				   COLUMN_LOCAL_FILE, downloader->filename,
+				   COLUMN_URI,        kz_downloader_get_uri(downloader),
+				   COLUMN_LOCAL_FILE, kz_downloader_get_filename(downloader),
 				   COLUMN_PROGRESS,   0.0,
 				   COLUMN_DOWNLOADER, downloader,
 				   TERMINATOR);
@@ -143,8 +143,8 @@ cb_downloader_added (KzDownloaderGroup *group,
 
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter,
-			   COLUMN_URI,        downloader->uri,
-			   COLUMN_LOCAL_FILE, downloader->filename,
+			   COLUMN_URI,        kz_downloader_get_uri(downloader),
+			   COLUMN_LOCAL_FILE, kz_downloader_get_filename(downloader),
 			   COLUMN_DOWNLOADER, downloader,
 			   TERMINATOR);
 }
