@@ -24,7 +24,6 @@
 
 #include <glib/gi18n.h>
 #include "kazehakase.h"
-#include "gobject-utils.h"
 #include "kz-thumbnails-view.h"
 #include "kz-paned.h"
 #include "kz-tab-label.h"
@@ -37,20 +36,18 @@ enum {
 
 
 /* Object class methods */
-static void     kz_thumbnails_sidebar_class_init   (KzThumbnailsSidebarClass *klass);
-static void     kz_thumbnails_sidebar_init         (KzThumbnailsSidebar *sidebar);
-static GObject *kz_thumbnails_sidebar_constructor  (GType type,
-						    guint n_props,
-						    GObjectConstructParam *props);
-static void     kz_thumbnails_sidebar_dispose      (GObject      *obj);
-static void     kz_thumbnails_sidebar_set_property (GObject      *object,
-						    guint         prop_id,
-						    const GValue *value,
-						    GParamSpec   *pspec);
-static void     kz_thumbnails_sidebar_get_property (GObject      *object,
-						    guint         prop_id,
-						    GValue       *value,
-						    GParamSpec   *pspec);
+static GObject *constructor  (GType type,
+                              guint n_props,
+                              GObjectConstructParam *props);
+static void     dispose      (GObject      *obj);
+static void     set_property (GObject      *object,
+                              guint         prop_id,
+                              const GValue *value,
+                              GParamSpec   *pspec);
+static void     get_property (GObject      *object,
+                              guint         prop_id,
+                              GValue       *value,
+                              GParamSpec   *pspec);
 static void     cb_paned_changed_position          (KzPaned *pane,
 						    GtkPositionType position,
 						    KzThumbnailsView *view);
@@ -70,10 +67,10 @@ static GtkWidget *kz_thumbnails_sidebar_history_search_new
 static KzSidebarEntry kz_sidebar_thumbnails[] = 
 {
 	{
-		priority_hint: 0,
-		label:         N_("Closed tabs"),
-		icon:          NULL,
-		create:        kz_thumbnails_sidebar_closed_tabs_new,
+		/* .priority_hint = */ 0,
+		/* .label         = */ N_("Closed tabs"),
+		/* .icon          = */ NULL,
+		/* .create        = */ kz_thumbnails_sidebar_closed_tabs_new,
 	}
 /*
 	{
@@ -85,8 +82,6 @@ static KzSidebarEntry kz_sidebar_thumbnails[] =
 */
 };
 
-static GtkVBoxClass *parent_class = NULL;
-
 
 KzSidebarEntry *
 kz_thumbnails_sidebar_get_entry (gint idx)
@@ -96,24 +91,19 @@ kz_thumbnails_sidebar_get_entry (gint idx)
 	return &kz_sidebar_thumbnails[idx];
 }
 
-
-KZ_OBJECT_GET_TYPE(kz_thumbnails_sidebar, "KzThumbnailsSidebar", KzThumbnailsSidebar,
-		   kz_thumbnails_sidebar_class_init, kz_thumbnails_sidebar_init,
-		   GTK_TYPE_VBOX)
-
+G_DEFINE_TYPE(KzThumbnailsSidebar, kz_thumbnails_sidebar, GTK_TYPE_VBOX)
 
 static void
 kz_thumbnails_sidebar_class_init (KzThumbnailsSidebarClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	parent_class = g_type_class_peek_parent(klass);
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
-	gobject_class->constructor  = kz_thumbnails_sidebar_constructor;
-	gobject_class->dispose      = kz_thumbnails_sidebar_dispose;
-	gobject_class->set_property = kz_thumbnails_sidebar_set_property;
-	gobject_class->get_property = kz_thumbnails_sidebar_get_property;
+	gobject_class->constructor  = constructor;
+	gobject_class->dispose      = dispose;
+	gobject_class->set_property = set_property;
+	gobject_class->get_property = get_property;
 
 	g_object_class_install_property
 		(gobject_class,
@@ -153,13 +143,13 @@ kz_thumbnails_sidebar_init (KzThumbnailsSidebar *sidebar)
 
 
 static GObject *
-kz_thumbnails_sidebar_constructor (GType                  type,
-			  	   guint                  n_props,
-			  	   GObjectConstructParam *props)
+constructor (GType                  type,
+             guint                  n_props,
+             GObjectConstructParam *props)
 {
 	KzThumbnailsSidebar *sidebar;
 	GObject *object;
-	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
+	GObjectClass *klass = G_OBJECT_CLASS(kz_thumbnails_sidebar_parent_class);
 
 	object = klass->constructor(type, n_props, props);
 
@@ -176,7 +166,7 @@ kz_thumbnails_sidebar_constructor (GType                  type,
 	return object;
 }
 static void
-kz_thumbnails_sidebar_dispose (GObject *obj)
+dispose (GObject *obj)
 {
 	KzThumbnailsSidebar *sidebar = KZ_THUMBNAILS_SIDEBAR(obj);
 
@@ -185,16 +175,16 @@ kz_thumbnails_sidebar_dispose (GObject *obj)
 
 	sidebar->sidebar = NULL;
 
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(obj);
+	if (G_OBJECT_CLASS (kz_thumbnails_sidebar_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_thumbnails_sidebar_parent_class)->dispose(obj);
 }
 
 
 static void
-kz_thumbnails_sidebar_set_property (GObject         *object,
-				    guint            prop_id,
-				    const GValue    *value,
-				    GParamSpec      *pspec)
+set_property (GObject         *object,
+              guint            prop_id,
+              const GValue    *value,
+              GParamSpec      *pspec)
 {
 	KzThumbnailsSidebar *sidebar = KZ_THUMBNAILS_SIDEBAR(object);
   
@@ -211,10 +201,10 @@ kz_thumbnails_sidebar_set_property (GObject         *object,
 
 
 static void
-kz_thumbnails_sidebar_get_property (GObject         *object,
-				    guint            prop_id,
-				    GValue          *value,
-				    GParamSpec      *pspec)
+get_property (GObject         *object,
+              guint            prop_id,
+              GValue          *value,
+              GParamSpec      *pspec)
 {
 	KzThumbnailsSidebar *sidebar = KZ_THUMBNAILS_SIDEBAR(object);
 
@@ -248,7 +238,7 @@ kz_thumbnails_sidebar_closed_tabs_new (KzSidebarEntry *entry,
 	proxy = kz_proxy_folder_new(sidebar->kz->closed_tabs);
 	
 	kz_thumbnails_view_set_folder(KZ_THUMBNAILS_VIEW(tsidebar->view),
-				      proxy);
+				      KZ_BOOKMARK_FOLDER(proxy));
 	g_object_unref(proxy);
 
 	return GTK_WIDGET(tsidebar);
@@ -299,24 +289,19 @@ cb_thumbnail_activate (KzThumbnailsView *view,
 		       gint index,
 		       KzWindow *kz)
 {
-	KzBookmark *tab;
+	KzBookmarkFolder *tab;
 	GtkWidget *kzembed;
 	KzTabLabel *kztab;
-	GList *list;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	list = kz_bookmark_get_children(kz->closed_tabs);
-	tab = KZ_BOOKMARK(g_list_nth_data(list, index));
-	g_list_free(list);
+	tab = KZ_BOOKMARK_FOLDER(kz_bookmark_folder_get_nth_child(kz->closed_tabs, index));
 	g_return_if_fail(tab);
 
 	kzembed = kz_window_open_new_tab(kz, NULL);
-	kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(
-			     GTK_NOTEBOOK(kz->notebook),
-			     kzembed));
+	kztab = kz_notebook_get_tab_label(KZ_NOTEBOOK(kz->notebook), KZ_EMBED(kzembed));
 	kz_tab_label_set_history(KZ_TAB_LABEL(kztab), tab);
 
-	kz_bookmark_remove(kz->closed_tabs, tab);
+	kz_bookmark_folder_remove(kz->closed_tabs, KZ_BOOKMARK(tab));
 }
