@@ -639,7 +639,6 @@ kz_utils_purge_files_by_time_stamp(const gchar *target_dir,
 		return;
 	}
 
-
 	fd = g_file_open_tmp("kzXXXXXX", &tmp_file, NULL);
 	if (fd == -1)
 		return;
@@ -666,7 +665,8 @@ kz_utils_purge_files_by_time_stamp(const gchar *target_dir,
 		time_int = atoi(time);
 		if (t - time_int < limit_seconds)
 		{
-			write(fd, buf, strlen(buf));
+			ssize_t size;
+			size = write(fd, buf, strlen(buf));
 			goto CLEANUP;
 		}
 
@@ -677,10 +677,11 @@ kz_utils_purge_files_by_time_stamp(const gchar *target_dir,
 		if (t - st.st_mtime < limit_seconds)
 		{
 			gchar *entry;
+			ssize_t size;
 
 			entry = g_strdup_printf(TIME_STAMP_FORMAT,
 						st.st_mtime, path);
-			write(fd, entry, strlen(entry));
+			size = write(fd, entry, strlen(entry));
 			goto CLEANUP;
 		}
 
@@ -761,13 +762,13 @@ ensure_encode_string(const gchar *text,
 }
 
 GTime 
-thumbnail_get_last_modified (const gchar *uri, EggPixbufThumbSize size)
+thumbnail_get_last_modified (const gchar *uri, EggPixbufThumbnailSize size)
 {
 	gchar *thumb_filename;
 	struct stat st;
 	int ret;
 
-	thumb_filename = egg_pixbuf_get_thumb_filename(uri, size);
+	thumb_filename = egg_pixbuf_get_thumbnail_filename(uri, size);
 
 	ret = g_stat(thumb_filename, &st);
 	g_free(thumb_filename);
@@ -1005,18 +1006,22 @@ kz_utils_get_system_name (void)
 	return system;
 }
 
-#ifdef G_OS_WIN32
 static gchar *win32_base_path = NULL;
 
 const gchar *
 kz_win32_base_path (void)
 {
+#ifndef G_OS_WIN32
+        return win32_base_path;
+#else
 	if (win32_base_path)
 		return win32_base_path;
-
-	win32_base_path = g_win32_get_package_installation_directory(PACKAGE,
-								     NULL);
+#if GLIB_CHECK_VERSION(2, 16, 0)
+	win32_base_path = g_win32_get_package_installation_directory_of_module(NULL);
+#else
+	win32_base_path = g_win32_get_package_installation_directory(PACKAGE, NULL);
+#endif
 
 	return win32_base_path;
-}
 #endif
+}
